@@ -78,8 +78,8 @@ std::string format_description =
     "     Irregular Sparse Matrices.\n"
     "csr: Compressed Sparse Row storage. Ginkgo implementation with\n"
     "     automatic strategy.\n"
-    "csrc: Ginkgo's CSR implementation with automatic stategy.\n"
-    "csri: Ginkgo's CSR implementation with inbalance strategy.\n"
+    "csrc: Ginkgo's CSR implementation with automatic strategy.\n"
+    "csri: Ginkgo's CSR implementation with imbalance strategy.\n"
     "csrm: Ginkgo's CSR implementation with merge_path strategy.\n"
     "csrs: Ginkgo's CSR implementation with sparselib strategy.\n"
     "ell: Ellpack format according to Bell and Garland: Efficient Sparse\n"
