@@ -41,10 +41,12 @@ OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 #include <array>
 #include <fstream>
 #include <functional>
+#include <iomanip>
 #include <map>
 #include <ostream>
 #include <random>
 #include <sstream>
+#include <stdexcept>
 #include <string>
 #include <type_traits>
 #include <utility>
@@ -52,10 +54,9 @@ OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
 
 #include <gflags/gflags.h>
-#include <rapidjson/document.h>
-#include <rapidjson/istreamwrapper.h>
-#include <rapidjson/ostreamwrapper.h>
-#include <rapidjson/prettywriter.h>
+
+
+#include <ginkgo/core/base/memory.hpp>
 
 
 #include "benchmark/utils/json.hpp"
@@ -69,6 +70,10 @@ DEFINE_string(executor, "reference",
               "The executor used to run the benchmarks, one of: reference, "
               "omp, cuda, hip");
 
+DEFINE_string(allocator, "default",
+              "The allocator used in the executor. Only relevant for CUDA and "
+              "HIP executors, one of: default, async, host, unified");
+
 DEFINE_uint32(device_id, 0, "ID of the device where to run the code");
 
 DEFINE_bool(overwrite, false,
@@ -92,10 +97,6 @@ DEFINE_string(
 DEFINE_bool(detailed, true,
             "If set, performs several runs to obtain more detailed results");
 
-DEFINE_bool(keep_errors, true,
-            "If set, writes exception messages during the execution into the "
-            "JSON output");
-
 DEFINE_bool(nested_names, false, "If set, separately logs nested operations");
 
 DEFINE_bool(profile, false,
@@ -137,6 +138,9 @@ DEFINE_double(
     "is lower than or equal to 1, the timing region is always 1 repetition.");
 
 
+std::unique_ptr<std::istream> input_stream;
+
+
 /**
  * Parses arguments through gflags and initialize a documentation string.
  *
@@ -146,27 +150,32 @@ DEFINE_double(
  * @param format  the format of the benchmark input data
  */
 void initialize_argument_parsing(int* argc, char** argv[], std::string& header,
-                                 std::string& format)
+                                 std::string& format, bool do_print = true)
 {
-    std::ostringstream doc;
-    doc << header << "Usage: " << (*argv)[0] << " [options]\n"
-        << format
-        << "  The results are written on standard output, in the same "
-           "format,\n"
-        << "  but with test cases extended to include an additional member "
-           "\n"
-        << "  object for each benchmark run.\n"
-        << "  If run with a --backup flag, an intermediate result is "
-           "written \n"
-        << "  to a file in the same format. The backup file can be used as "
-           "\n"
-        << "  input to this test suite, and the benchmarking will \n"
-        << "  continue from the point where the backup file was created.";
+    if (do_print) {
+        std::ostringstream doc;
+        doc << header << "Usage: " << (*argv)[0] << " [options]\n"
+            << format
+            << "  The results are written on standard output, in the same "
+               "format,\n"
+            << "  but with test cases extended to include an additional member "
+               "\n"
+            << "  object for each benchmark run.\n"
+            << "  If run with a --backup flag, an intermediate result is "
+               "written \n"
+            << "  to a file in the same format. The backup file can be used as "
+               "\n"
+            << "  input to this test suite, and the benchmarking will \n"
+            << "  continue from the point where the backup file was created.";
 
-    gflags::SetUsageMessage(doc.str());
-    std::ostringstream ver;
-    ver << gko::version_info::get();
-    gflags::SetVersionString(ver.str());
+        gflags::SetUsageMessage(doc.str());
+        std::ostringstream ver;
+        ver << gko::version_info::get();
+        gflags::SetVersionString(ver.str());
+    } else {
+        gflags::SetUsageMessage("");
+        gflags::SetVersionString("");
+    }
     gflags::ParseCommandLineFlags(argc, argv, true);
     if (FLAGS_profile) {
         FLAGS_repetitions = "1";
@@ -176,10 +185,18 @@ void initialize_argument_parsing(int* argc, char** argv[], std::string& header,
             FLAGS_profiler_hook = "auto";
         }
     }
+    std::string input_str(FLAGS_input);
+    if (!input_str.empty()) {
+        if (input_str.back() == ']') {
+            input_stream = std::make_unique<std::stringstream>(input_str);
+        } else {
+            input_stream = std::make_unique<std::ifstream>(input_str);
+        }
+    }
 }
 
 /**
- * Print general benchmark informations using the common available parameters
+ * Print general benchmark information using the common available parameters
  *
  * @param extra  describes benchmark specific extra parameters to output
  */
@@ -187,20 +204,19 @@ void print_general_information(const std::string& extra)
 {
     std::clog << gko::version_info::get() << std::endl
               << "Running on " << FLAGS_executor << "(" << FLAGS_device_id
-              << ")" << std::endl
+              << ")\n"
               << "Running with " << FLAGS_warmup << " warm iterations and ";
     if (FLAGS_repetitions == "auto") {
         std::clog << "adaptively determined repetititions with "
                   << FLAGS_min_repetitions
                   << " <= rep <= " << FLAGS_max_repetitions
-                  << " and a minimal runtime of " << FLAGS_min_runtime << "s"
-                  << std::endl;
+                  << " and a minimal runtime of " << FLAGS_min_runtime << "s\n";
     } else {
-        std::clog << FLAGS_repetitions << " running iterations" << std::endl;
+        std::clog << FLAGS_repetitions << " running iterations\n";
     }
     std::clog << "The random seed for right hand sides is " << FLAGS_seed
-              << std::endl
-              << extra;
+              << '\n'
+              << extra << std::endl;
 }
 
 
@@ -229,32 +245,19 @@ std::shared_ptr<gko::log::ProfilerHook> create_profiler_hook(
 }
 
 
-struct owning_profiling_scope_guard {
-    std::string name;
-    gko::log::profiling_scope_guard guard;
-
-    owning_profiling_scope_guard() = default;
-
-    owning_profiling_scope_guard(std::string name_,
-                                 gko::log::ProfilerHook* profiler_hook)
-        : name(std::move(name_)), guard{profiler_hook->user_range(name.c_str())}
-    {}
-};
-
-
 struct annotate_functor {
-    owning_profiling_scope_guard operator()(std::string name) const
+    gko::log::profiling_scope_guard operator()(const char* name) const
     {
         if (profiler_hook) {
-            return owning_profiling_scope_guard{std::move(name),
-                                                profiler_hook.get()};
+            return profiler_hook->user_range(name);
         }
         return {};
     }
 
-    gko::log::profiling_scope_guard operator()(const char* name) const
+    gko::log::profiling_scope_guard operator()(const char* name,
+                                               bool should_annotate) const
     {
-        if (profiler_hook) {
+        if (profiler_hook && should_annotate) {
             return profiler_hook->user_range(name);
         }
         return {};
@@ -292,25 +295,15 @@ std::vector<std::string> split(const std::string& s, char delimiter = ',')
 // returns the stream to be used as input of the application
 std::istream& get_input_stream()
 {
-    static auto stream = []() -> std::unique_ptr<std::istream> {
-        std::string input_str(FLAGS_input);
-        if (input_str.empty()) {
-            return nullptr;
-        }
-        if (input_str.back() == ']') {
-            return std::make_unique<std::stringstream>(input_str);
-        }
-        return std::make_unique<std::ifstream>(input_str);
-    }();
-    if (stream) {
-        return *stream;
+    if (input_stream) {
+        return *input_stream;
     }
     return std::cin;
 }
 
 
 // backup generation
-void backup_results(rapidjson::Document& results)
+void backup_results(json& results)
 {
     static int next = 0;
     static auto filenames = []() -> std::array<std::string, 2> {
@@ -329,6 +322,40 @@ void backup_results(rapidjson::Document& results)
 }
 
 
+inline std::shared_ptr<gko::CudaAllocatorBase> create_cuda_allocator()
+{
+    std::string flag{FLAGS_allocator};
+    if (flag == "default") {
+        return std::make_shared<gko::CudaAllocator>();
+    } else if (flag == "async") {
+        return std::make_shared<gko::CudaAsyncAllocator>(nullptr);
+    } else if (flag == "unified") {
+        return std::make_shared<gko::CudaUnifiedAllocator>(FLAGS_device_id);
+    } else if (flag == "host") {
+        return std::make_shared<gko::CudaHostAllocator>(FLAGS_device_id);
+    } else {
+        throw std::runtime_error{"Unknown allocator type " + flag};
+    }
+}
+
+
+inline std::shared_ptr<gko::HipAllocatorBase> create_hip_allocator()
+{
+    std::string flag{FLAGS_allocator};
+    if (flag == "default") {
+        return std::make_shared<gko::HipAllocator>();
+    } else if (flag == "async") {
+        return std::make_shared<gko::HipAsyncAllocator>(nullptr);
+    } else if (flag == "unified") {
+        return std::make_shared<gko::HipUnifiedAllocator>(FLAGS_device_id);
+    } else if (flag == "host") {
+        return std::make_shared<gko::HipHostAllocator>(FLAGS_device_id);
+    } else {
+        throw std::runtime_error{"Unknown allocator type " + flag};
+    }
+}
+
+
 // executor mapping
 const std::map<std::string, std::function<std::shared_ptr<gko::Executor>(bool)>>
     executor_factory{
@@ -337,12 +364,14 @@ const std::map<std::string, std::function<std::shared_ptr<gko::Executor>(bool)>>
         {"cuda",
          [](bool) {
              return gko::CudaExecutor::create(FLAGS_device_id,
-                                              gko::OmpExecutor::create(), true);
+                                              gko::OmpExecutor::create(),
+                                              create_cuda_allocator());
          }},
         {"hip",
          [](bool) {
              return gko::HipExecutor::create(FLAGS_device_id,
-                                             gko::OmpExecutor::create(), true);
+                                             gko::OmpExecutor::create(),
+                                             create_hip_allocator());
          }},
         {"dpcpp", [](bool use_gpu_timer) {
              auto property = dpcpp_queue_property::in_order;
@@ -368,16 +397,17 @@ const std::map<std::string,
          [](MPI_Comm comm) {
              FLAGS_device_id = gko::experimental::mpi::map_rank_to_device_id(
                  comm, gko::CudaExecutor::get_num_devices());
-             return gko::CudaExecutor::create(
-                 FLAGS_device_id, gko::ReferenceExecutor::create(), false,
-                 gko::allocation_mode::device);
+             return gko::CudaExecutor::create(FLAGS_device_id,
+                                              gko::ReferenceExecutor::create(),
+                                              create_cuda_allocator());
          }},
         {"hip",
          [](MPI_Comm comm) {
              FLAGS_device_id = gko::experimental::mpi::map_rank_to_device_id(
                  comm, gko::HipExecutor::get_num_devices());
-             return gko::HipExecutor::create(
-                 FLAGS_device_id, gko::ReferenceExecutor::create(), true);
+             return gko::HipExecutor::create(FLAGS_device_id,
+                                             gko::ReferenceExecutor::create(),
+                                             create_hip_allocator());
          }},
         {"dpcpp", [](MPI_Comm comm) {
              if (gko::DpcppExecutor::get_num_devices("gpu")) {
@@ -530,279 +560,4 @@ gko::remove_complex<ValueType> compute_max_relative_norm2(
 }
 
 
-/**
- * A class for controlling the number warmup and timed iterations.
- *
- * The behavior is determined by the following flags
- * - 'repetitions' switch between fixed and adaptive number of iterations
- * - 'warmup' warmup iterations, applies in fixed and adaptive case
- * - 'min_repetitions' minimal number of repetitions (adaptive case)
- * - 'max_repetitions' maximal number of repetitions (adaptive case)
- * - 'min_runtime' minimal total runtime (adaptive case)
- * - 'repetition_growth_factor' controls the increase between two successive
- *   timings
- *
- * Usage:
- * `IterationControl` exposes the member functions:
- * - `warmup_run()`: controls run defined by `warmup` flag
- * - `run(bool)`: controls run defined by all other flags
- * - `get_timer()`: access to underlying timer
- * The first two methods return an object that is to be used in a range-based
- * for loop:
- * ```
- * IterationControl ic(get_timer(...));
- *
- * // warmup run always uses fixed number of iteration and does not issue
- * // timings
- * for(auto status: ic.warmup_run()){
- *   // execute benchmark
- * }
- * // run may use adaptive number of iterations (depending on cmd line flag)
- * // and issues timing (unless manage_timings is false)
- * for(auto status: ic.run(manage_timings [default is true])){
- *   if(! manage_timings) ic.get_timer->tic();
- *   // execute benchmark
- *   if(! manage_timings) ic.get_timer->toc();
- * }
- *
- * ```
- * At the beginning of both methods, the timer is reset.
- * The `status` object exposes the member
- * - `cur_it`, containing the current iteration number,
- * and the methods
- * - `is_finished`, checks if the benchmark is finished,
- */
-class IterationControl {
-    using IndexType = unsigned int;  //!< to be compatible with GFLAGS type
-
-    class run_control;
-
-public:
-    /**
-     * Creates an `IterationControl` object.
-     *
-     * Uses the commandline flags to setup the stopping criteria for the
-     * warmup and timed run.
-     *
-     * @param timer  the timer that is to be used for the timings
-     */
-    explicit IterationControl(const std::shared_ptr<Timer>& timer)
-    {
-        status_warmup_ = {TimerManager{timer, false}, FLAGS_warmup,
-                          FLAGS_warmup, 0., 0};
-        if (FLAGS_repetitions == "auto") {
-            status_run_ = {TimerManager{timer, true}, FLAGS_min_repetitions,
-                           FLAGS_max_repetitions, FLAGS_min_runtime};
-        } else {
-            const auto reps =
-                static_cast<unsigned int>(std::stoi(FLAGS_repetitions));
-            status_run_ = {TimerManager{timer, true}, reps, reps, 0., 0};
-        }
-    }
-
-    IterationControl() = default;
-    IterationControl(const IterationControl&) = default;
-    IterationControl(IterationControl&&) = default;
-
-    /**
-     * Creates iterable `run_control` object for the warmup run.
-     *
-     * This run uses always a fixed number of iterations.
-     */
-    run_control warmup_run()
-    {
-        status_warmup_.cur_it = 0;
-        status_warmup_.managed_timer.clear();
-        return run_control{&status_warmup_};
-    }
-
-    /**
-     * Creates iterable `run_control` object for the timed run.
-     *
-     * This run may be adaptive, depending on the commandline flags.
-     *
-     * @param manage_timings If true, the timer calls (`tic/toc`) are handled
-     * by the `run_control` object, otherwise they need to be executed outside
-     */
-    run_control run(bool manage_timings = true)
-    {
-        status_run_.cur_it = 0;
-        status_run_.managed_timer.clear();
-        status_run_.managed_timer.manage_timings = manage_timings;
-        return run_control{&status_run_};
-    }
-
-    std::shared_ptr<Timer> get_timer() const
-    {
-        return status_run_.managed_timer.timer;
-    }
-
-    /**
-     * Compute the time from the given statistical method
-     *
-     * @param method  the statistical method. If the timer does not have the
-     *                same iteration as the IterationControl, it can only use
-     *                average from the IterationControl.
-     *
-     * @return the statistical time
-     */
-    double compute_time(const std::string& method = "average") const
-    {
-        if (status_run_.managed_timer.timer->get_num_repetitions() ==
-            this->get_num_repetitions()) {
-            return status_run_.managed_timer.compute_time(method);
-        } else {
-            assert(method == "average");
-            return status_run_.managed_timer.get_total_time() /
-                   this->get_num_repetitions();
-        }
-    }
-
-    IndexType get_num_repetitions() const { return status_run_.cur_it; }
-
-private:
-    struct TimerManager {
-        std::shared_ptr<Timer> timer;
-        bool manage_timings = false;
-
-        void tic()
-        {
-            if (manage_timings) {
-                timer->tic();
-            }
-        }
-        void toc(unsigned int num = 1)
-        {
-            if (manage_timings) {
-                timer->toc(num);
-            }
-        }
-
-        void clear() { timer->clear(); }
-
-        double get_total_time() const { return timer->get_total_time(); }
-
-        double compute_time(const std::string& method = "average") const
-        {
-            return timer->compute_time(method);
-        }
-    };
-
-    /**
-     * Stores stopping criteria of the adaptive benchmark run as well as the
-     * current iteration number.
-     */
-    struct status {
-        TimerManager managed_timer{};
-
-        IndexType min_it = 0;
-        IndexType max_it = 0;
-        double max_runtime = 0.;
-
-        IndexType cur_it = 0;
-
-        /**
-         * checks if the adaptive run is complete
-         *
-         * the adaptive run is complete if:
-         * - the minimum number of iteration is reached
-         * - and either:
-         *   - the maximum number of repetitions is reached
-         *   - the total runtime is above the threshold
-         *
-         * @return completeness state of the adaptive run
-         */
-        bool is_finished() const
-        {
-            return cur_it >= min_it &&
-                   (cur_it >= max_it ||
-                    managed_timer.get_total_time() >= max_runtime);
-        }
-    };
-
-    /**
-     * Iterable class managing the benchmark iteration.
-     *
-     * Has to be used in a range-based for loop.
-     */
-    struct run_control {
-        struct iterator {
-            /**
-             * Increases the current iteration count and finishes timing if
-             * necessary.
-             *
-             * As `++it` is the last step of a for-loop, the managed_timer is
-             * stopped, if enough iterations have passed since the last timing.
-             * The interval between two timings is steadily increased to
-             * reduce the timing overhead.
-             */
-            iterator operator++()
-            {
-                cur_info->cur_it++;
-                if (cur_info->cur_it >= next_timing && !stopped) {
-                    cur_info->managed_timer.toc(
-                        static_cast<unsigned>(cur_info->cur_it - start_timing));
-                    stopped = true;
-                    next_timing = static_cast<IndexType>(std::ceil(
-                        next_timing * FLAGS_repetition_growth_factor));
-                    // If repetition_growth_factor <= 1, next_timing will be
-                    // next iteration.
-                    if (next_timing <= cur_info->cur_it) {
-                        next_timing = cur_info->cur_it + 1;
-                    }
-                }
-                return *this;
-            }
-
-            status operator*() const { return *cur_info; }
-
-            /**
-             * Checks if the benchmark is finished and handles timing, if
-             * necessary.
-             *
-             * As `begin != end` is the first step in a for-loop, the
-             * managed_timer is started, if it was previously stopped.
-             * Additionally, if the benchmark is complete and the managed_timer
-             * is still running it is stopped. (This may occur if the maximal
-             * number of repetitions is surpassed)
-             *
-             * Uses only the information from the `status` object, i.e.
-             * the right hand side is ignored.
-             *
-             * @return true if benchmark is not finished, else false
-             */
-            bool operator!=(const iterator&)
-            {
-                const bool is_finished = cur_info->is_finished();
-                if (!is_finished && stopped) {
-                    stopped = false;
-                    cur_info->managed_timer.tic();
-                    start_timing = cur_info->cur_it;
-                } else if (is_finished && !stopped) {
-                    cur_info->managed_timer.toc(
-                        static_cast<unsigned>(cur_info->cur_it - start_timing));
-                    stopped = true;
-                }
-                return !is_finished;
-            }
-
-            status* cur_info;
-            IndexType next_timing = 1;   //!< next iteration to stop timing
-            IndexType start_timing = 0;  //!< iteration for starting timing
-            bool stopped = true;
-        };
-
-        iterator begin() const { return iterator{info}; }
-
-        // not used, could potentially be used in c++17 as a sentinel
-        iterator end() const { return iterator{}; }
-
-        status* info;
-    };
-
-    status status_warmup_;
-    status status_run_;
-};
-
-
 #endif  // GKO_BENCHMARK_UTILS_GENERAL_HPP_
