@@ -33,6 +33,9 @@ OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 #include <ginkgo/core/preconditioner/ic.hpp>
 
 
+GKO_BEGIN_DISABLE_DEPRECATION_WARNINGS
+
+
 #include <memory>
 
 
@@ -44,6 +47,9 @@ OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 #include <ginkgo/core/solver/bicgstab.hpp>
 
 
+#include "core/test/utils.hpp"
+
+
 namespace {
 
 
@@ -77,9 +83,8 @@ TEST_F(IcFactory, KnowsItsExecutor)
 
 TEST_F(IcFactory, CanSetLSolverFactory)
 {
-    auto ic_factory = ic_prec_type::build()
-                          .with_l_solver_factory(this->l_factory)
-                          .on(this->exec);
+    auto ic_factory =
+        ic_prec_type::build().with_l_solver(this->l_factory).on(this->exec);
 
     ASSERT_EQ(ic_factory->get_parameters().l_solver_factory, this->l_factory);
 }
@@ -88,7 +93,7 @@ TEST_F(IcFactory, CanSetLSolverFactory)
 TEST_F(IcFactory, CanSetFactorizationFactory)
 {
     auto ic_factory = ic_prec_type::build()
-                          .with_factorization_factory(this->fact_factory)
+                          .with_factorization(this->fact_factory)
                           .on(this->exec);
 
     ASSERT_EQ(ic_factory->get_parameters().factorization_factory,
@@ -96,4 +101,34 @@ TEST_F(IcFactory, CanSetFactorizationFactory)
 }
 
 
+TEST_F(IcFactory, DeprecatedFactoryParameter)
+{
+    auto ilu_factory = ic_prec_type::build()
+                           .with_l_solver_factory(this->l_factory)
+                           .with_factorization_factory(this->fact_factory)
+                           .on(this->exec);
+
+    ASSERT_EQ(ilu_factory->get_parameters().l_solver_factory, this->l_factory);
+    ASSERT_EQ(ilu_factory->get_parameters().factorization_factory,
+              this->fact_factory);
+}
+
+
+TEST_F(IcFactory, DeferredFactoryParameter)
+{
+    auto ic_factory = ic_prec_type::build()
+                          .with_l_solver(solver_type::build())
+                          .with_factorization(ic_type::build())
+                          .on(this->exec);
+
+    GKO_ASSERT_DYNAMIC_TYPE(ic_factory->get_parameters().l_solver_factory,
+                            solver_type::Factory);
+    GKO_ASSERT_DYNAMIC_TYPE(ic_factory->get_parameters().factorization_factory,
+                            ic_type::Factory);
+}
+
+
 }  // namespace
+
+
+GKO_END_DISABLE_DEPRECATION_WARNINGS
