@@ -64,10 +64,9 @@ protected:
           idr_factory(
               Solver::build()
                   .with_criteria(
-                      gko::stop::Iteration::build().with_max_iters(3u).on(exec),
+                      gko::stop::Iteration::build().with_max_iters(3u),
                       gko::stop::ResidualNorm<value_type>::build()
-                          .with_reduction_factor(gko::remove_complex<T>{1e-6})
-                          .on(exec))
+                          .with_reduction_factor(gko::remove_complex<T>{1e-6}))
                   .on(exec)),
           solver(idr_factory->generate(mtx))
     {}
@@ -162,14 +161,9 @@ TYPED_TEST(Idr, CanSetPreconditionerGenerator)
     using value_type = typename TestFixture::value_type;
     auto idr_factory =
         Solver::build()
-            .with_criteria(
-                gko::stop::Iteration::build().with_max_iters(3u).on(this->exec))
-            .with_preconditioner(
-                Solver::build()
-                    .with_criteria(
-                        gko::stop::Iteration::build().with_max_iters(3u).on(
-                            this->exec))
-                    .on(this->exec))
+            .with_criteria(gko::stop::Iteration::build().with_max_iters(3u))
+            .with_preconditioner(Solver::build().with_criteria(
+                gko::stop::Iteration::build().with_max_iters(3u)))
             .on(this->exec);
 
     auto solver = idr_factory->generate(this->mtx);
@@ -209,15 +203,13 @@ TYPED_TEST(Idr, CanSetPreconditionerInFactory)
     using Solver = typename TestFixture::Solver;
     std::shared_ptr<Solver> idr_precond =
         Solver::build()
-            .with_criteria(
-                gko::stop::Iteration::build().with_max_iters(3u).on(this->exec))
+            .with_criteria(gko::stop::Iteration::build().with_max_iters(3u))
             .on(this->exec)
             ->generate(this->mtx);
 
     auto idr_factory =
         Solver::build()
-            .with_criteria(
-                gko::stop::Iteration::build().with_max_iters(3u).on(this->exec))
+            .with_criteria(gko::stop::Iteration::build().with_max_iters(3u))
             .with_generated_preconditioner(idr_precond)
             .on(this->exec);
     auto solver = idr_factory->generate(this->mtx);
@@ -236,15 +228,13 @@ TYPED_TEST(Idr, ThrowsOnWrongPreconditionerInFactory)
         Mtx::create(this->exec, gko::dim<2>{2, 2});
     std::shared_ptr<Solver> idr_precond =
         Solver::build()
-            .with_criteria(
-                gko::stop::Iteration::build().with_max_iters(3u).on(this->exec))
+            .with_criteria(gko::stop::Iteration::build().with_max_iters(3u))
             .on(this->exec)
             ->generate(wrong_sized_mtx);
 
     auto idr_factory =
         Solver::build()
-            .with_criteria(
-                gko::stop::Iteration::build().with_max_iters(3u).on(this->exec))
+            .with_criteria(gko::stop::Iteration::build().with_max_iters(3u))
             .with_generated_preconditioner(idr_precond)
             .on(this->exec);
 
@@ -257,15 +247,13 @@ TYPED_TEST(Idr, CanSetPreconditioner)
     using Solver = typename TestFixture::Solver;
     std::shared_ptr<Solver> idr_precond =
         Solver::build()
-            .with_criteria(
-                gko::stop::Iteration::build().with_max_iters(3u).on(this->exec))
+            .with_criteria(gko::stop::Iteration::build().with_max_iters(3u))
             .on(this->exec)
             ->generate(this->mtx);
 
     auto idr_factory =
         Solver::build()
-            .with_criteria(
-                gko::stop::Iteration::build().with_max_iters(3u).on(this->exec))
+            .with_criteria(gko::stop::Iteration::build().with_max_iters(3u))
             .on(this->exec);
     auto solver = idr_factory->generate(this->mtx);
     solver->set_preconditioner(idr_precond);
@@ -283,8 +271,7 @@ TYPED_TEST(Idr, CanSetSubspaceDim)
     auto idr_factory =
         Solver::build()
             .with_subspace_dim(8u)
-            .with_criteria(
-                gko::stop::Iteration::build().with_max_iters(4u).on(this->exec))
+            .with_criteria(gko::stop::Iteration::build().with_max_iters(4u))
             .on(this->exec);
     auto solver = idr_factory->generate(this->mtx);
     auto subspace_dim = solver->get_subspace_dim();
@@ -320,8 +307,7 @@ TYPED_TEST(Idr, CanSetKappa)
     auto idr_factory =
         Solver::build()
             .with_kappa(real_type{0.05})
-            .with_criteria(
-                gko::stop::Iteration::build().with_max_iters(4u).on(this->exec))
+            .with_criteria(gko::stop::Iteration::build().with_max_iters(4u))
             .on(this->exec);
     auto solver = idr_factory->generate(this->mtx);
     auto kappa = solver->get_kappa();
@@ -359,8 +345,7 @@ TYPED_TEST(Idr, CanSetDeterministic)
     auto idr_factory =
         Solver::build()
             .with_deterministic(true)
-            .with_criteria(
-                gko::stop::Iteration::build().with_max_iters(4u).on(this->exec))
+            .with_criteria(gko::stop::Iteration::build().with_max_iters(4u))
             .on(this->exec);
     auto solver = idr_factory->generate(this->mtx);
     auto deterministic = solver->get_deterministic();
@@ -396,8 +381,7 @@ TYPED_TEST(Idr, CanSetComplexSubspace)
     auto idr_factory =
         Solver::build()
             .with_complex_subspace(true)
-            .with_criteria(
-                gko::stop::Iteration::build().with_max_iters(4u).on(this->exec))
+            .with_criteria(gko::stop::Iteration::build().with_max_iters(4u))
             .on(this->exec);
     auto solver = idr_factory->generate(this->mtx);
     auto complex_subspace = solver->get_complex_subspace();
@@ -420,10 +404,27 @@ TYPED_TEST(Idr, CanSetComplexSubspaceAgain)
 
     auto solver = idr_factory->generate(this->mtx);
 
-    solver->set_complex_subpsace(false);
+    solver->set_complex_subspace(false);
 
     ASSERT_EQ(solver->get_complex_subspace(), false);
 }
 
 
+TYPED_TEST(Idr, PassExplicitFactory)
+{
+    using Solver = typename TestFixture::Solver;
+    auto stop_factory = gko::share(
+        gko::stop::Iteration::build().with_max_iters(1u).on(this->exec));
+    auto precond_factory = gko::share(Solver::build().on(this->exec));
+
+    auto factory = Solver::build()
+                       .with_criteria(stop_factory)
+                       .with_preconditioner(precond_factory)
+                       .on(this->exec);
+
+    ASSERT_EQ(factory->get_parameters().criteria.front(), stop_factory);
+    ASSERT_EQ(factory->get_parameters().preconditioner, precond_factory);
+}
+
+
 }  // namespace
