@@ -43,17 +43,21 @@ OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 #include <initializer_list>
 #include <string>
 #include <type_traits>
+#include <typeinfo>
 
 
 #include <gtest/gtest.h>
 
 
 #include <ginkgo/core/base/array.hpp>
+#include <ginkgo/core/base/batch_multi_vector.hpp>
 #include <ginkgo/core/base/math.hpp>
 #include <ginkgo/core/base/mtx_io.hpp>
+#include <ginkgo/core/base/name_demangling.hpp>
 #include <ginkgo/core/matrix/dense.hpp>
 
 
+#include "core/base/batch_utilities.hpp"
 #include "core/base/extended_float.hpp"
 
 
@@ -315,6 +319,86 @@ double get_relative_error(const MatrixData1& first, const MatrixData2& second)
 }
 
 
+template <typename MatrixData1, typename MatrixData2>
+::testing::AssertionResult batch_matrices_near_impl(
+    const std::string& first_expression, const std::string& second_expression,
+    const std::string& tolerance_expression, const MatrixData1& first,
+    const MatrixData2& second, double tolerance)
+{
+    std::vector<double> err;
+    for (size_type b = 0; b < first.size(); ++b) {
+        if (first.size() != second.size()) {
+            return ::testing::AssertionFailure()
+                   << "Expected matrices of equal size\n\t" << first_expression
+                   << " is of size [" << first[b].size[0] << " x "
+                   << first[b].size[1] << "]\n\t" << second_expression
+                   << " is of size [" << second[b].size[0] << " x "
+                   << second[b].size[1] << "]"
+                   << " for batch " << b;
+        }
+
+        err.push_back(detail::get_relative_error(first[b], second[b]));
+    }
+
+    auto bat = std::find_if(err.begin(), err.end(),
+                            [&](double& e) { return !(e <= tolerance); });
+    if (bat == err.end()) {
+        return ::testing::AssertionSuccess();
+    } else {
+        const auto b_pos = static_cast<ptrdiff_t>(bat - err.begin());
+        auto num_rows = first[b_pos].size[0];
+        auto num_cols = first[b_pos].size[1];
+        auto fail = ::testing::AssertionFailure();
+        fail << "Error for batch: " << b_pos << "\n Relative error between "
+             << first_expression << " and " << second_expression << " is "
+             << err[b_pos] << "\n"
+             << "\twhich is larger than " << tolerance_expression
+             << " (which is " << tolerance << ")\n";
+        if (num_rows * num_cols <= 1000) {
+            fail << first_expression << " is:\n";
+            detail::print_matrix(fail, first[b_pos]);
+            fail << second_expression << " is:\n";
+            detail::print_matrix(fail, second[b_pos]);
+            fail << "component-wise relative error is:\n";
+            detail::print_componentwise_error(fail, first[b_pos],
+                                              second[b_pos]);
+        } else {
+            // build output filenames
+            auto test_case_info =
+                ::testing::UnitTest::GetInstance()->current_test_info();
+            auto testname =
+                test_case_info ? std::string{test_case_info->test_case_name()} +
+                                     "." + test_case_info->name()
+                               : std::string{"null"};
+            auto firstfile = testname + "." + first_expression + ".mtx";
+            auto secondfile = testname + "." + second_expression + ".mtx";
+            auto to_remove = [](char c) {
+                return !std::isalnum(c) && c != '_' && c != '.' && c != '-' &&
+                       c != '<' && c != '>';
+            };
+            // remove all but alphanumerical and _.-<> characters from
+            // expressions
+            firstfile.erase(
+                std::remove_if(firstfile.begin(), firstfile.end(), to_remove),
+                firstfile.end());
+            secondfile.erase(
+                std::remove_if(secondfile.begin(), secondfile.end(), to_remove),
+                secondfile.end());
+            // save matrices
+            std::ofstream first_stream{firstfile};
+            gko::write_raw(first_stream, first[b_pos],
+                           gko::layout_type::coordinate);
+            std::ofstream second_stream{secondfile};
+            gko::write_raw(second_stream, second[b_pos],
+                           gko::layout_type::coordinate);
+            fail << first_expression << " saved as " << firstfile << "\n";
+            fail << second_expression << " saved as " << secondfile << "\n";
+        }
+        return fail;
+    }
+}
+
+
 template <typename MatrixData1, typename MatrixData2>
 ::testing::AssertionResult matrices_near_impl(
     const std::string& first_expression, const std::string& second_expression,
@@ -600,6 +684,81 @@ template <>
 }
 
 
+/**
+ * This is a gtest predicate which checks if two batch matrices are relatively
+ * near.
+ *
+ * More formally, it checks whether the following equation holds for each of the
+ * matrices in the batch:
+ *
+ * ```
+ * ||first - second|| <= tolerance * max(||first||, ||second||)
+ * ```
+ *
+ * This function should not be called directly, but used in conjunction with
+ * `ASSERT_PRED_FORMAT3` as follows:
+ *
+ * ```
+ * // Check if first and second are near
+ * ASSERT_PRED_FORMAT3(gko::test::assertions::batch_matrices_near,
+ *                     first, second, tolerance);
+ * // Check if first and second are far
+ * ASSERT_PRED_FORMAT3(!gko::test::assertions::batch_matrices_near,
+ *                     first, second, tolerance);
+ * ```
+ *
+ * @see GKO_ASSERT_BATCH_MTX_NEAR
+ * @see GKO_EXPECT_BATCH_MTX_NEAR
+ */
+template <typename Mat1, typename Mat2>
+::testing::AssertionResult batch_matrices_near(
+    const std::string& first_expression, const std::string& second_expression,
+    const std::string& tolerance_expression, const Mat1* first,
+    const Mat2* second, double tolerance)
+{
+    auto exec = first->get_executor()->get_master();
+    using value_type1 = typename Mat1::value_type;
+    using value_type2 = typename Mat2::value_type;
+
+    auto first_data = gko::batch::write<value_type1, int, Mat1>(first);
+    auto second_data = gko::batch::write<value_type2, int, Mat2>(second);
+
+    if (first_data.size() != second_data.size()) {
+        return ::testing::AssertionFailure()
+               << "Expected same batch sizes for " << first_expression
+               << " and " << second_expression << ", but got batch size "
+               << first_data.size() << " for " << first_expression
+               << " and batch size " << second_data.size() << " for "
+               << second_expression;
+    }
+
+    for (size_type b = 0; b < first_data.size(); ++b) {
+        first_data[b].ensure_row_major_order();
+        second_data[b].ensure_row_major_order();
+    }
+
+    return detail::batch_matrices_near_impl(
+        detail::remove_pointer_wrapper(first_expression),
+        detail::remove_pointer_wrapper(second_expression), tolerance_expression,
+        first_data, second_data, tolerance);
+}
+
+
+template <typename Mat1, typename T>
+::testing::AssertionResult batch_matrices_near(
+    const std::string& first_expression, const std::string& second_expression,
+    const std::string& tolerance_expression, const Mat1* first,
+    std::initializer_list<std::initializer_list<T>> second, double tolerance)
+{
+    auto second_mtx =
+        batch::initialize<batch::MultiVector<detail::remove_container<T>>>(
+            second, first->get_executor()->get_master());
+    return batch_matrices_near(
+        first_expression, detail::remove_list_wrapper(second_expression),
+        tolerance_expression, first, second_mtx.get(), tolerance);
+}
+
+
 /**
  * This is a gtest predicate which checks if two matrices are relatively near.
  *
@@ -853,6 +1012,45 @@ template <typename LinOp1, typename T>
 }
 
 
+template <typename Ptr1, typename Ptr2>
+::testing::AssertionResult dynamic_type_eq(const std::string& expr1,
+                                           const std::string& expr2,
+                                           const Ptr1& ptr1, const Ptr2& ptr2)
+{
+    auto& ref1 = *ptr1;
+    auto& ref2 = *ptr2;
+    if (typeid(ref1) == typeid(ref2)) {
+        return ::testing::AssertionSuccess();
+    } else {
+        return ::testing::AssertionFailure()
+               << "mismatching dynamic types\n"
+               << expr1 << " is\n\t"
+               << gko::name_demangling::get_type_name(typeid(ref1)) << "\n"
+               << expr2 << " is\n\t"
+               << gko::name_demangling::get_type_name(typeid(ref2)) << "\n";
+    }
+}
+
+
+template <typename Ptr>
+::testing::AssertionResult dynamic_type_is(const std::string& expr,
+                                           const std::string&, const Ptr& ptr,
+                                           const std::type_info& type)
+{
+    auto& ref = *ptr;
+    if (typeid(ref) == type) {
+        return ::testing::AssertionSuccess();
+    } else {
+        return ::testing::AssertionFailure()
+               << "unexpected dynamic type\n"
+               << expr << " is\n\t"
+               << gko::name_demangling::get_type_name(typeid(ref)) << "\n"
+               << "but we expected\n\t"
+               << gko::name_demangling::get_type_name(type) << "\n";
+    }
+}
+
+
 namespace detail {
 
 
@@ -940,6 +1138,44 @@ T* plain_ptr(T* ptr)
     }
 
 
+/**
+ * Checks if two batched matrices are near each other.
+ *
+ * More formally, it checks whether the following equation holds:
+ *
+ * ```
+ * ||_mtx1 - _mtx2|| <= _tol * max(||_mtx1||, ||_mtx2||)
+ * ```
+ * for all batches
+ *
+ * Has to be called from within a google test unit test.
+ * Internally calls gko::test::assertions::batch_matrices_near().
+ *
+ * @param _mtx1  first matrix
+ * @param _mtx2  second matrix
+ * @param _tol  tolerance level
+ */
+#define GKO_ASSERT_BATCH_MTX_NEAR(_mtx1, _mtx2, _tol)                     \
+    {                                                                     \
+        using ::gko::test::assertions::detail::l;                         \
+        using ::gko::test::assertions::detail::plain_ptr;                 \
+        ASSERT_PRED_FORMAT3(::gko::test::assertions::batch_matrices_near, \
+                            plain_ptr(_mtx1), plain_ptr(_mtx2), _tol);    \
+    }
+
+
+/**
+ * @copydoc GKO_ASSERT_MTX_NEAR
+ */
+#define GKO_EXPECT_BATCH_MTX_NEAR(_mtx1, _mtx2, _tol)                     \
+    {                                                                     \
+        using ::gko::test::assertions::detail::l;                         \
+        using ::gko::test::assertions::detail::plain_ptr;                 \
+        EXPECT_PRED_FORMAT3(::gko::test::assertions::batch_matrices_near, \
+                            plain_ptr(_mtx1), plain_ptr(_mtx2), _tol);    \
+    }
+
+
 /**
  * Checks if two matrices are near each other.
  *
@@ -1054,4 +1290,51 @@ T* plain_ptr(T* ptr)
     }
 
 
+/**
+ * Checks if the dynamic types of the objects referenced by two pointers are
+ * equal.
+ *
+ * @param _ptr1  the first pointer
+ * @param _ptr2  the second pointer
+ */
+#define GKO_ASSERT_DYNAMIC_TYPE_EQ(_ptr1, _ptr2)                             \
+    {                                                                        \
+        ASSERT_PRED_FORMAT2(::gko::test::assertions::dynamic_type_eq, _ptr1, \
+                            _ptr2);                                          \
+    }
+
+
+/**
+ * @copydoc GKO_ASSERT_DYNAMIC_TYPE_EQ
+ */
+#define GKO_EXPECT_DYNAMIC_TYPE_EQ(_ptr1, _ptr2)                             \
+    {                                                                        \
+        EXPECT_PRED_FORMAT2(::gko::test::assertions::dynamic_type_eq, _ptr1, \
+                            _ptr2);                                          \
+    }
+
+
+/**
+ * Checks if the dynamic type of a pointer to an object matches a given type
+ *
+ * @param _ptr  the pointer
+ * @param _type  the expected type
+ */
+#define GKO_ASSERT_DYNAMIC_TYPE(_ptr, _type)                                \
+    {                                                                       \
+        ASSERT_PRED_FORMAT2(::gko::test::assertions::dynamic_type_is, _ptr, \
+                            typeid(_type));                                 \
+    }
+
+
+/**
+ * @copydoc GKO_ASSERT_DYNAMIC_TYPE
+ */
+#define GKO_EXPECT_DYNAMIC_TYPE(_ptr, _type)                                \
+    {                                                                       \
+        EXPECT_PRED_FORMAT2(::gko::test::assertions::dynamic_type_is, _ptr, \
+                            typeid(_type));                                 \
+    }
+
+
 #endif  // GKO_CORE_TEST_UTILS_ASSERTIONS_HPP_
