@@ -38,33 +38,27 @@ OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
 
 #include <ginkgo/core/base/executor.hpp>
+#include <ginkgo/core/base/stream.hpp>
 
 
+#include "core/test/gtest/resources.hpp"
 #include "cuda/base/device.hpp"
 
 
 namespace {
 
 
-class CudaEnvironment : public ::testing::Environment {
-public:
-    void TearDown() override { gko::kernels::cuda::reset_device(0); }
-};
-
-testing::Environment* cuda_env =
-    testing::AddGlobalTestEnvironment(new CudaEnvironment);
-
-
 class CudaTestFixture : public ::testing::Test {
 protected:
     CudaTestFixture()
         : ref(gko::ReferenceExecutor::create()),
 #ifdef GKO_TEST_NONDEFAULT_STREAM
-          exec(gko::CudaExecutor::create(
-              0, ref, false, gko::default_cuda_alloc_mode, stream.get()))
-#else
-          exec(gko::CudaExecutor::create(0, ref))
+          stream(ResourceEnvironment::cuda_device_id),
 #endif
+          exec(gko::CudaExecutor::create(
+              ResourceEnvironment::cuda_device_id, ref,
+              std::make_shared<gko::CudaAllocator>(), stream.get())),
+          guard(exec->get_scoped_device_id_guard())
     {}
 
     void TearDown()
@@ -75,11 +69,10 @@ protected:
         }
     }
 
-#ifdef GKO_TEST_NONDEFAULT_STREAM
     gko::cuda_stream stream;
-#endif
     std::shared_ptr<gko::ReferenceExecutor> ref;
     std::shared_ptr<gko::CudaExecutor> exec;
+    gko::scoped_device_id_guard guard;
 };
 
 
