@@ -255,13 +255,12 @@ int main(int argc, char* argv[])
             {"omp", [] { return gko::OmpExecutor::create(); }},
             {"cuda",
              [] {
-                 return gko::CudaExecutor::create(0, gko::OmpExecutor::create(),
-                                                  true);
+                 return gko::CudaExecutor::create(0,
+                                                  gko::OmpExecutor::create());
              }},
             {"hip",
              [] {
-                 return gko::HipExecutor::create(0, gko::OmpExecutor::create(),
-                                                 true);
+                 return gko::HipExecutor::create(0, gko::OmpExecutor::create());
              }},
             {"dpcpp",
              [] {
@@ -292,12 +291,10 @@ int main(int argc, char* argv[])
     const RealValueType reduction_factor{1e-7};
     // Generate solver and solve the system
     cg::build()
-        .with_criteria(gko::stop::Iteration::build()
-                           .with_max_iters(discretization_points)
-                           .on(exec),
-                       gko::stop::ResidualNorm<ValueType>::build()
-                           .with_reduction_factor(reduction_factor)
-                           .on(exec))
+        .with_criteria(
+            gko::stop::Iteration::build().with_max_iters(discretization_points),
+            gko::stop::ResidualNorm<ValueType>::build().with_reduction_factor(
+                reduction_factor))
         .on(exec)
         // notice how our custom StencilMatrix can be used in the same way as
         // any built-in type
