@@ -72,13 +72,12 @@ int main(int argc, char* argv[])
             {"omp", [] { return gko::OmpExecutor::create(); }},
             {"cuda",
              [] {
-                 return gko::CudaExecutor::create(0, gko::OmpExecutor::create(),
-                                                  true);
+                 return gko::CudaExecutor::create(0,
+                                                  gko::OmpExecutor::create());
              }},
             {"hip",
              [] {
-                 return gko::HipExecutor::create(0, gko::OmpExecutor::create(),
-                                                 true);
+                 return gko::HipExecutor::create(0, gko::OmpExecutor::create());
              }},
             {"dpcpp",
              [] {
@@ -119,12 +118,10 @@ int main(int argc, char* argv[])
     // Generate solver operator  (A - zI)^-1
     auto solver =
         solver_type::build()
-            .with_criteria(gko::stop::Iteration::build()
-                               .with_max_iters(system_max_iterations)
-                               .on(exec),
+            .with_criteria(gko::stop::Iteration::build().with_max_iters(
+                               system_max_iterations),
                            gko::stop::ResidualNorm<precision>::build()
-                               .with_reduction_factor(system_residual_goal)
-                               .on(exec))
+                               .with_reduction_factor(system_residual_goal))
             .on(exec)
             ->generate(system_matrix);
 
