@@ -68,13 +68,12 @@ int main(int argc, char* argv[])
             {"omp", [] { return gko::OmpExecutor::create(); }},
             {"cuda",
              [] {
-                 return gko::CudaExecutor::create(0, gko::OmpExecutor::create(),
-                                                  true);
+                 return gko::CudaExecutor::create(0,
+                                                  gko::OmpExecutor::create());
              }},
             {"hip",
              [] {
-                 return gko::HipExecutor::create(0, gko::OmpExecutor::create(),
-                                                 true);
+                 return gko::HipExecutor::create(0, gko::OmpExecutor::create());
              }},
             {"dpcpp",
              [] {
@@ -114,19 +113,13 @@ int main(int argc, char* argv[])
     RealValueType inner_reduction_factor{1e-2};
     auto solver_gen =
         ir::build()
-            .with_solver(
-                cg::build()
-                    .with_criteria(
-                        gko::stop::ResidualNorm<ValueType>::build()
-                            .with_reduction_factor(inner_reduction_factor)
-                            .on(exec))
-                    .on(exec))
-            .with_criteria(
-                gko::stop::Iteration::build().with_max_iters(max_iters).on(
-                    exec),
+            .with_solver(cg::build().with_criteria(
                 gko::stop::ResidualNorm<ValueType>::build()
-                    .with_reduction_factor(outer_reduction_factor)
-                    .on(exec))
+                    .with_reduction_factor(inner_reduction_factor)))
+            .with_criteria(
+                gko::stop::Iteration::build().with_max_iters(max_iters),
+                gko::stop::ResidualNorm<ValueType>::build()
+                    .with_reduction_factor(outer_reduction_factor))
             .on(exec);
     // Create solver
     auto solver = solver_gen->generate(A);
