@@ -371,13 +371,12 @@ int main(int argc, char* argv[])
             {"omp", [] { return gko::OmpExecutor::create(); }},
             {"cuda",
              [] {
-                 return gko::CudaExecutor::create(0, gko::OmpExecutor::create(),
-                                                  true);
+                 return gko::CudaExecutor::create(0,
+                                                  gko::OmpExecutor::create());
              }},
             {"hip",
              [] {
-                 return gko::HipExecutor::create(0, gko::OmpExecutor::create(),
-                                                 true);
+                 return gko::HipExecutor::create(0, gko::OmpExecutor::create());
              }},
             {"dpcpp",
              [] {
@@ -417,10 +416,8 @@ int main(int argc, char* argv[])
         solver::build()
             .with_criteria(
                 gko::stop::ResidualNorm<ValueType>::build()
-                    .with_reduction_factor(reduction_factor)
-                    .on(exec),
-                gko::stop::Iteration::build().with_max_iters(max_iters).on(
-                    exec))
+                    .with_reduction_factor(reduction_factor),
+                gko::stop::Iteration::build().with_max_iters(max_iters))
             .with_preconditioner(preconditioner::create(exec))
             .on(exec);
 
