@@ -39,7 +39,11 @@ OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
 
 #include <hip/hip_runtime.h>
+#if HIP_VERSION >= 50200000
+#include <hipsparse/hipsparse.h>
+#else
 #include <hipsparse.h>
+#endif
 
 
 #include <ginkgo/core/base/exception_helpers.hpp>
