@@ -188,6 +188,27 @@ GKO_ATTRIBUTES constexpr GKO_INLINE
            equal_dimensions<CurrentDimension + 1>(first, second);
 }
 
+/**
+ * Helper that stores the first type of a parameter pack, if its length is
+ * greater 0.
+ */
+template <class...>
+struct head;
+
+/**
+ * @copydoc head
+ */
+template <class First, class... Rest>
+struct head<First, Rest...> {
+    using type = First;
+};
+
+/**
+ * @copydoc head
+ */
+template <class... T>
+using head_t = typename head<T...>::type;
+
 
 }  // namespace detail
 
@@ -255,7 +276,7 @@ GKO_ATTRIBUTES constexpr GKO_INLINE
  * `x` an `y` are ranges, and `alpha` is a scalar.
  * Range operations are optimized for memory access, and the above code does not
  * allocate additional storage for intermediate ranges `alpha * x`
- * or `aplha * x + y`. In fact, the entire computation is done during the
+ * or `alpha * x + y`. In fact, the entire computation is done during the
  * assignment, and the results of operations `+` and `*` only register the data,
  * and the types of operations that will be computed once the results are
  * needed.
@@ -274,7 +295,7 @@ GKO_ATTRIBUTES constexpr GKO_INLINE
  *
  * __`mmul` is not a highly-optimized BLAS-3 version of the matrix
  * multiplication.__ The current design of ranges and accessors prevents that,
- * so if you need a high-perfromance matrix multiplication, you should use one
+ * so if you need a high-performance matrix multiplication, you should use one
  * of the libraries that provide that, or implement your own
  * (you can use pointwise range operations to help simplify that). However,
  * range design might get improved in the future to allow efficient
@@ -327,7 +348,12 @@ public:
      *
      * @param params  parameters forwarded to Accessor constructor.
      */
-    template <typename... AccessorParams>
+    template <
+        typename... AccessorParams,
+        typename = std::enable_if_t<
+            sizeof...(AccessorParams) != 1 ||
+            !std::is_same<
+                range, std::decay<detail::head_t<AccessorParams...>>>::value>>
     GKO_ATTRIBUTES constexpr explicit range(AccessorParams&&... params)
         : accessor_{std::forward<AccessorParams>(params)...}
     {}
@@ -588,6 +614,17 @@ struct implement_binary_operation<operation_kind::range_by_scalar,
 
 }  // namespace detail
 
+#define GKO_DEPRECATED_UNARY_RANGE_OPERATION(_operation_deprecated_name,     \
+                                             _operation_name)                \
+    namespace accessor {                                                     \
+    template <typename Operand>                                              \
+    struct GKO_DEPRECATED("Please use " #_operation_name)                    \
+        _operation_deprecated_name : _operation_name<Operand> {};            \
+    }                                                                        \
+    static_assert(true,                                                      \
+                  "This assert is used to counter the false positive extra " \
+                  "semi-colon warnings")
+
 
 #define GKO_ENABLE_UNARY_RANGE_OPERATION(_operation_name, _operator_name, \
                                          _operator)                       \
@@ -682,21 +719,30 @@ GKO_ENABLE_UNARY_RANGE_OPERATION(bitwise_not, operator~,
                                  accessor::detail::bitwise_not);
 
 // common unary functions
+
 GKO_ENABLE_UNARY_RANGE_OPERATION(zero_operation, zero,
                                  accessor::detail::zero_operation);
-GKO_ENABLE_UNARY_RANGE_OPERATION(one_operaton, one,
+GKO_ENABLE_UNARY_RANGE_OPERATION(one_operation, one,
                                  accessor::detail::one_operation);
-GKO_ENABLE_UNARY_RANGE_OPERATION(abs_operaton, abs,
+GKO_ENABLE_UNARY_RANGE_OPERATION(abs_operation, abs,
                                  accessor::detail::abs_operation);
-GKO_ENABLE_UNARY_RANGE_OPERATION(real_operaton, real,
+GKO_ENABLE_UNARY_RANGE_OPERATION(real_operation, real,
                                  accessor::detail::real_operation);
-GKO_ENABLE_UNARY_RANGE_OPERATION(imag_operaton, imag,
+GKO_ENABLE_UNARY_RANGE_OPERATION(imag_operation, imag,
                                  accessor::detail::imag_operation);
-GKO_ENABLE_UNARY_RANGE_OPERATION(conj_operaton, conj,
+GKO_ENABLE_UNARY_RANGE_OPERATION(conj_operation, conj,
                                  accessor::detail::conj_operation);
-GKO_ENABLE_UNARY_RANGE_OPERATION(squared_norm_operaton, squared_norm,
+GKO_ENABLE_UNARY_RANGE_OPERATION(squared_norm_operation, squared_norm,
                                  accessor::detail::squared_norm_operation);
 
+GKO_DEPRECATED_UNARY_RANGE_OPERATION(one_operaton, one_operation);
+GKO_DEPRECATED_UNARY_RANGE_OPERATION(abs_operaton, abs_operation);
+GKO_DEPRECATED_UNARY_RANGE_OPERATION(real_operaton, real_operation);
+GKO_DEPRECATED_UNARY_RANGE_OPERATION(imag_operaton, imag_operation);
+GKO_DEPRECATED_UNARY_RANGE_OPERATION(conj_operaton, conj_operation);
+GKO_DEPRECATED_UNARY_RANGE_OPERATION(squared_norm_operaton,
+                                     squared_norm_operation);
+
 namespace accessor {
 
 
@@ -740,6 +786,7 @@ struct transpose_operation {
 GKO_BIND_UNARY_RANGE_OPERATION_TO_OPERATOR(transpose_operation, transpose);
 
 
+#undef GKO_DEPRECATED_UNARY_RANGE_OPERATION
 #undef GKO_DEFINE_SIMPLE_UNARY_OPERATION
 #undef GKO_ENABLE_UNARY_RANGE_OPERATION
 
@@ -815,6 +862,9 @@ GKO_BIND_UNARY_RANGE_OPERATION_TO_OPERATOR(transpose_operation, transpose);
                   "semi-colon warnings")
 
 
+#define GKO_DEPRECATED_SIMPLE_BINARY_OPERATION(_deprecated_name, _name) \
+    struct GKO_DEPRECATED("Please use " #_name) _deprecated_name : _name {}
+
 #define GKO_DEFINE_SIMPLE_BINARY_OPERATION(_name, ...)                         \
     struct _name {                                                             \
     private:                                                                   \
@@ -893,6 +943,8 @@ GKO_DEFINE_SIMPLE_BINARY_OPERATION(right_shift, first >> second);
 GKO_DEFINE_SIMPLE_BINARY_OPERATION(max_operation, max(first, second));
 GKO_DEFINE_SIMPLE_BINARY_OPERATION(min_operation, min(first, second));
 
+GKO_DEPRECATED_SIMPLE_BINARY_OPERATION(max_operaton, max_operation);
+GKO_DEPRECATED_SIMPLE_BINARY_OPERATION(min_operaton, min_operation);
 }  // namespace detail
 }  // namespace accessor
 
@@ -935,9 +987,9 @@ GKO_ENABLE_BINARY_RANGE_OPERATION(right_shift, operator>>,
                                   accessor::detail::right_shift);
 
 // common binary functions
-GKO_ENABLE_BINARY_RANGE_OPERATION(max_operaton, max,
+GKO_ENABLE_BINARY_RANGE_OPERATION(max_operation, max,
                                   accessor::detail::max_operation);
-GKO_ENABLE_BINARY_RANGE_OPERATION(min_operaton, min,
+GKO_ENABLE_BINARY_RANGE_OPERATION(min_operation, min,
                                   accessor::detail::min_operation);
 
 
