@@ -128,6 +128,16 @@ constexpr bool less_equal(const T&& lhs, const T&& rhs)
 }
 
 
+// available in <type_traits> with C++17
+template <class...>
+struct conjunction : std::true_type {};
+template <class B1>
+struct conjunction<B1> : B1 {};
+template <class B1, class... Bn>
+struct conjunction<B1, Bn...>
+    : std::conditional_t<bool(B1::value), conjunction<Bn...>, B1> {};
+
+
 }  // namespace xstd
 }  // namespace gko
 
