@@ -294,9 +294,9 @@ inline typename std::remove_reference<OwningPointer>::type&& give(
  *       same as calling .get() on the smart pointer.
  */
 template <typename Pointer>
-[[deprecated("no longer necessary, just pass the object without lend")]] inline
-    typename std::enable_if<detail::have_ownership_s<Pointer>::value,
-                            detail::pointee<Pointer>*>::type
+GKO_DEPRECATED("no longer necessary, just pass the object without lend")
+inline typename std::enable_if<detail::have_ownership_s<Pointer>::value,
+                               detail::pointee<Pointer>*>::type
     lend(const Pointer& p)
 {
     return p.get();
@@ -313,9 +313,9 @@ template <typename Pointer>
  *       returns `p`.
  */
 template <typename Pointer>
-[[deprecated("no longer necessary, just pass the object without lend")]] inline
-    typename std::enable_if<!detail::have_ownership_s<Pointer>::value,
-                            detail::pointee<Pointer>*>::type
+GKO_DEPRECATED("no longer necessary, just pass the object without lend")
+inline typename std::enable_if<!detail::have_ownership_s<Pointer>::value,
+                               detail::pointee<Pointer>*>::type
     lend(const Pointer& p)
 {
     return p;
