@@ -1,5 +1,5 @@
 /*******************************<GINKGO LICENSE>******************************
-Copyright (c) 2017-2021, the Ginkgo authors
+Copyright (c) 2017-2023, the Ginkgo authors
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without
@@ -47,9 +47,8 @@ OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 #include <ginkgo/core/base/std_extensions.hpp>
 #include <ginkgo/core/factorization/par_ilu.hpp>
 #include <ginkgo/core/matrix/dense.hpp>
-#include <ginkgo/core/solver/lower_trs.hpp>
 #include <ginkgo/core/solver/solver_traits.hpp>
-#include <ginkgo/core/solver/upper_trs.hpp>
+#include <ginkgo/core/solver/triangular.hpp>
 #include <ginkgo/core/stop/combined.hpp>
 #include <ginkgo/core/stop/iteration.hpp>
 #include <ginkgo/core/stop/residual_norm.hpp>
@@ -67,11 +66,10 @@ namespace preconditioner {
  * It allows to set both the solver for L and the solver for U independently,
  * while providing the defaults solver::LowerTrs and solver::UpperTrs, which
  * are direct triangular solvers.
- * For these solvers, a factory can be provided (with `with_l_solver_factory`
- * and `with_u_solver_factory`) to have more control over their behavior.
- * In particular, it is possible to use an iterative method for solving the
- * triangular systems. The default parameters for an iterative triangluar
- * solver are:
+ * For these solvers, a factory can be provided (with `with_l_solver` and
+ * `with_u_solver`) to have more control over their behavior. In particular, it
+ * is possible to use an iterative method for solving the triangular systems.
+ * The default parameters for an iterative triangluar solver are:
  * - reduction factor = 1e-4
  * - max iteration = <number of rows of the matrix given to the solver>
  * Solvers without such criteria can also be used, in which case none are set.
@@ -132,25 +130,102 @@ public:
         Ilu<typename USolverType::transposed_type,
             typename LSolverType::transposed_type, ReverseApply, IndexType>;
 
-    GKO_CREATE_FACTORY_PARAMETERS(parameters, Factory)
-    {
+    class Factory;
+
+    struct parameters_type
+        : public enable_parameters_type<parameters_type, Factory> {
         /**
          * Factory for the L solver
          */
-        std::shared_ptr<typename l_solver_type::Factory>
-            GKO_FACTORY_PARAMETER_SCALAR(l_solver_factory, nullptr);
+        std::shared_ptr<const typename l_solver_type::Factory>
+            l_solver_factory{};
 
         /**
          * Factory for the U solver
          */
-        std::shared_ptr<typename u_solver_type::Factory>
-            GKO_FACTORY_PARAMETER_SCALAR(u_solver_factory, nullptr);
+        std::shared_ptr<const typename u_solver_type::Factory>
+            u_solver_factory{};
 
         /**
          * Factory for the factorization
          */
-        std::shared_ptr<LinOpFactory> GKO_FACTORY_PARAMETER_SCALAR(
-            factorization_factory, nullptr);
+        std::shared_ptr<const LinOpFactory> factorization_factory{};
+
+        GKO_DEPRECATED("use with_l_solver instead")
+        parameters_type& with_l_solver_factory(
+            deferred_factory_parameter<const typename l_solver_type::Factory>
+                solver)
+        {
+            return with_l_solver(std::move(solver));
+        }
+
+        parameters_type& with_l_solver(
+            deferred_factory_parameter<const typename l_solver_type::Factory>
+                solver)
+        {
+            this->l_solver_generator = std::move(solver);
+            this->deferred_factories["l_solver"] = [](const auto& exec,
+                                                      auto& params) {
+                if (!params.l_solver_generator.is_empty()) {
+                    params.l_solver_factory =
+                        params.l_solver_generator.on(exec);
+                }
+            };
+            return *this;
+        }
+
+        GKO_DEPRECATED("use with_u_solver instead")
+        parameters_type& with_u_solver_factory(
+            deferred_factory_parameter<const typename u_solver_type::Factory>
+                solver)
+        {
+            return with_u_solver(std::move(solver));
+        }
+
+        parameters_type& with_u_solver(
+            deferred_factory_parameter<const typename u_solver_type::Factory>
+                solver)
+        {
+            this->u_solver_generator = std::move(solver);
+            this->deferred_factories["u_solver"] = [](const auto& exec,
+                                                      auto& params) {
+                if (!params.u_solver_generator.is_empty()) {
+                    params.u_solver_factory =
+                        params.u_solver_generator.on(exec);
+                }
+            };
+            return *this;
+        }
+
+        GKO_DEPRECATED("use with_factorization instead")
+        parameters_type& with_factorization_factory(
+            deferred_factory_parameter<const LinOpFactory> factorization)
+        {
+            return with_factorization(std::move(factorization));
+        }
+
+        parameters_type& with_factorization(
+            deferred_factory_parameter<const LinOpFactory> factorization)
+        {
+            this->factorization_generator = std::move(factorization);
+            this->deferred_factories["factorization"] = [](const auto& exec,
+                                                           auto& params) {
+                if (!params.factorization_generator.is_empty()) {
+                    params.factorization_factory =
+                        params.factorization_generator.on(exec);
+                }
+            };
+            return *this;
+        }
+
+    private:
+        deferred_factory_parameter<const typename l_solver_type::Factory>
+            l_solver_generator;
+
+        deferred_factory_parameter<const typename u_solver_type::Factory>
+            u_solver_generator;
+
+        deferred_factory_parameter<const LinOpFactory> factorization_generator;
     };
 
     GKO_ENABLE_LIN_OP_FACTORY(Ilu, parameters, Factory);
@@ -206,43 +281,99 @@ public:
         return std::move(transposed);
     }
 
+    /**
+     * Copy-assigns an ILU preconditioner. Preserves the executor,
+     * shallow-copies the solvers and parameters. Creates a clone of the solvers
+     * if they are on the wrong executor.
+     */
+    Ilu& operator=(const Ilu& other)
+    {
+        if (&other != this) {
+            EnableLinOp<Ilu>::operator=(other);
+            auto exec = this->get_executor();
+            l_solver_ = other.l_solver_;
+            u_solver_ = other.u_solver_;
+            parameters_ = other.parameters_;
+            if (other.get_executor() != exec) {
+                l_solver_ = gko::clone(exec, l_solver_);
+                u_solver_ = gko::clone(exec, u_solver_);
+            }
+        }
+        return *this;
+    }
+
+    /**
+     * Move-assigns an ILU preconditioner. Preserves the executor,
+     * moves the solvers and parameters. Creates a clone of the solvers
+     * if they are on the wrong executor. The moved-from object is empty (0x0
+     * with nullptr solvers and default parameters)
+     */
+    Ilu& operator=(Ilu&& other)
+    {
+        if (&other != this) {
+            EnableLinOp<Ilu>::operator=(other);
+            auto exec = this->get_executor();
+            l_solver_ = std::move(other.l_solver_);
+            u_solver_ = std::move(other.u_solver_);
+            parameters_ = std::exchange(other.parameters_, parameters_type{});
+            if (other.get_executor() != exec) {
+                l_solver_ = gko::clone(exec, l_solver_);
+                u_solver_ = gko::clone(exec, u_solver_);
+            }
+        }
+        return *this;
+    }
+
+    /**
+     * Copy-constructs an ILU preconditioner. Inherits the executor,
+     * shallow-copies the solvers and parameters.
+     */
+    Ilu(const Ilu& other) : Ilu{other.get_executor()} { *this = other; }
+
+    /**
+     * Move-constructs an ILU preconditioner. Inherits the executor,
+     * moves the solvers and parameters. The moved-from object is empty (0x0
+     * with nullptr solvers and default parameters)
+     */
+    Ilu(Ilu&& other) : Ilu{other.get_executor()} { *this = std::move(other); }
+
 protected:
-    void apply_impl(const LinOp *b, LinOp *x) const override
+    void apply_impl(const LinOp* b, LinOp* x) const override
     {
         // take care of real-to-complex apply
         precision_dispatch_real_complex<value_type>(
             [&](auto dense_b, auto dense_x) {
                 this->set_cache_to(dense_b);
                 if (!ReverseApply) {
-                    l_solver_->apply(dense_b, cache_.intermediate.get());
+                    l_solver_->apply(dense_b, cache_.intermediate);
                     if (u_solver_->apply_uses_initial_guess()) {
-                        dense_x->copy_from(cache_.intermediate.get());
+                        dense_x->copy_from(cache_.intermediate);
                     }
-                    u_solver_->apply(cache_.intermediate.get(), dense_x);
+                    u_solver_->apply(cache_.intermediate, dense_x);
                 } else {
-                    u_solver_->apply(dense_b, cache_.intermediate.get());
+                    u_solver_->apply(dense_b, cache_.intermediate);
                     if (l_solver_->apply_uses_initial_guess()) {
-                        dense_x->copy_from(cache_.intermediate.get());
+                        dense_x->copy_from(cache_.intermediate);
                     }
-                    l_solver_->apply(cache_.intermediate.get(), dense_x);
+                    l_solver_->apply(cache_.intermediate, dense_x);
                 }
             },
             b, x);
     }
 
-    void apply_impl(const LinOp *alpha, const LinOp *b, const LinOp *beta,
-                    LinOp *x) const override
+    void apply_impl(const LinOp* alpha, const LinOp* b, const LinOp* beta,
+                    LinOp* x) const override
     {
         precision_dispatch_real_complex<value_type>(
             [&](auto dense_alpha, auto dense_b, auto dense_beta, auto dense_x) {
                 this->set_cache_to(dense_b);
                 if (!ReverseApply) {
-                    l_solver_->apply(dense_b, cache_.intermediate.get());
-                    u_solver_->apply(dense_alpha, cache_.intermediate.get(),
+                    l_solver_->apply(dense_b, cache_.intermediate);
+                    u_solver_->apply(dense_alpha, cache_.intermediate,
                                      dense_beta, dense_x);
                 } else {
-                    u_solver_->apply(dense_b, cache_.intermediate.get());
-                    l_solver_->apply(dense_alpha, cache_.intermediate.get(),
+                    u_solver_->apply(dense_b, cache_.intermediate);
+                    l_solver_->apply(dense_alpha, cache_.intermediate,
                                      dense_beta, dense_x);
                 }
             },
@@ -253,7 +384,7 @@ protected:
         : EnableLinOp<Ilu>(std::move(exec))
     {}
 
-    explicit Ilu(const Factory *factory, std::shared_ptr<const LinOp> lin_op)
+    explicit Ilu(const Factory* factory, std::shared_ptr<const LinOp> lin_op)
         : EnableLinOp<Ilu>(factory->get_executor(), lin_op->get_size()),
           parameters_{factory->get_parameters()}
     {
@@ -309,7 +440,7 @@ protected:
      * @param b  Right hand side of the first solve. Also acts as the initial
      *           guess, meaning the intermediate value will be a copy of b
      */
-    void set_cache_to(const LinOp *b) const
+    void set_cache_to(const LinOp* b) const
     {
         if (cache_.intermediate == nullptr) {
             cache_.intermediate =
@@ -330,20 +461,18 @@ protected:
     template <typename SolverType>
     static std::enable_if_t<solver::has_with_criteria<SolverType>::value,
                             std::unique_ptr<SolverType>>
-    generate_default_solver(const std::shared_ptr<const Executor> &exec,
-                            const std::shared_ptr<const LinOp> &mtx)
+    generate_default_solver(const std::shared_ptr<const Executor>& exec,
+                            const std::shared_ptr<const LinOp>& mtx)
     {
         constexpr gko::remove_complex<value_type> default_reduce_residual{1e-4};
         const unsigned int default_max_iters{
             static_cast<unsigned int>(mtx->get_size()[0])};
 
         return SolverType::build()
-            .with_criteria(gko::stop::Iteration::build()
-                               .with_max_iters(default_max_iters)
-                               .on(exec),
-                           gko::stop::ResidualNorm<value_type>::build()
-                               .with_reduction_factor(default_reduce_residual)
-                               .on(exec))
+            .with_criteria(
+                gko::stop::Iteration::build().with_max_iters(default_max_iters),
+                gko::stop::ResidualNorm<value_type>::build()
+                    .with_reduction_factor(default_reduce_residual))
             .on(exec)
             ->generate(mtx);
     }
@@ -354,8 +483,8 @@ protected:
     template <typename SolverType>
     static std::enable_if_t<!solver::has_with_criteria<SolverType>::value,
                             std::unique_ptr<SolverType>>
-    generate_default_solver(const std::shared_ptr<const Executor> &exec,
-                            const std::shared_ptr<const LinOp> &mtx)
+    generate_default_solver(const std::shared_ptr<const Executor>& exec,
+                            const std::shared_ptr<const LinOp>& mtx)
     {
         return SolverType::build().on(exec)->generate(mtx);
     }
@@ -376,10 +505,10 @@ private:
     mutable struct cache_struct {
         cache_struct() = default;
         ~cache_struct() = default;
-        cache_struct(const cache_struct &) {}
-        cache_struct(cache_struct &&) {}
-        cache_struct &operator=(const cache_struct &) { return *this; }
-        cache_struct &operator=(cache_struct &&) { return *this; }
+        cache_struct(const cache_struct&) {}
+        cache_struct(cache_struct&&) {}
+        cache_struct& operator=(const cache_struct&) { return *this; }
+        cache_struct& operator=(cache_struct&&) { return *this; }
         std::unique_ptr<LinOp> intermediate{};
     } cache_;
 };
