@@ -177,19 +177,15 @@ public:
      */
     Ir(Ir&&);
 
-    GKO_CREATE_FACTORY_PARAMETERS(parameters, Factory)
-    {
-        /**
-         * Criterion factories.
-         */
-        std::vector<std::shared_ptr<const stop::CriterionFactory>>
-            GKO_FACTORY_PARAMETER_VECTOR(criteria, nullptr);
+    class Factory;
 
+    struct parameters_type
+        : enable_iterative_solver_factory_parameters<parameters_type, Factory> {
         /**
          * Inner solver factory.
          */
-        std::shared_ptr<const LinOpFactory> GKO_FACTORY_PARAMETER_SCALAR(
-            solver, nullptr);
+        std::shared_ptr<const LinOpFactory> GKO_DEFERRED_FACTORY_PARAMETER(
+            solver);
 
         /**
          * Already generated solver. If one is provided, the factory `solver`
@@ -306,7 +302,7 @@ struct workspace_traits<Ir<ValueType>> {
  * limited stop criterion(iterations and relacation_factor).
  *
  * @param factory  the shared pointer of factory
- * @param iteration  the maximum number of iteraion, which default is 1
+ * @param iteration  the maximum number of iteration, which default is 1
  * @param relaxation_factor  the relaxation factor for Richardson
  *
  * @return the pointer of Ir(Richardson)
@@ -319,8 +315,7 @@ auto build_smoother(std::shared_ptr<const LinOpFactory> factory,
     return Ir<ValueType>::build()
         .with_solver(factory)
         .with_relaxation_factor(relaxation_factor)
-        .with_criteria(
-            gko::stop::Iteration::build().with_max_iters(iteration).on(exec))
+        .with_criteria(gko::stop::Iteration::build().with_max_iters(iteration))
         .on(exec);
 }
 
@@ -329,7 +324,7 @@ auto build_smoother(std::shared_ptr<const LinOpFactory> factory,
  * limited stop criterion(iterations and relacation_factor).
  *
  * @param solver  the shared pointer of solver
- * @param iteration  the maximum number of iteraion, which default is 1
+ * @param iteration  the maximum number of iteration, which default is 1
  * @param relaxation_factor  the relaxation factor for Richardson
  *
  * @return the pointer of Ir(Richardson)
@@ -344,8 +339,7 @@ auto build_smoother(std::shared_ptr<const LinOp> solver,
     return Ir<ValueType>::build()
         .with_generated_solver(solver)
         .with_relaxation_factor(relaxation_factor)
-        .with_criteria(
-            gko::stop::Iteration::build().with_max_iters(iteration).on(exec))
+        .with_criteria(gko::stop::Iteration::build().with_max_iters(iteration))
         .on(exec);
 }
 
