@@ -1,5 +1,5 @@
 /*******************************<GINKGO LICENSE>******************************
-Copyright (c) 2017-2021, the Ginkgo authors
+Copyright (c) 2017-2023, the Ginkgo authors
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without
@@ -49,7 +49,7 @@ OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
 
 #include "core/base/utils.hpp"
-#include "core/components/prefix_sum.hpp"
+#include "core/components/prefix_sum_kernels.hpp"
 #include "core/matrix/coo_builder.hpp"
 #include "core/matrix/csr_builder.hpp"
 #include "omp/components/csr_spgeam.hpp"
@@ -68,10 +68,10 @@ namespace par_ilut_factorization {
 
 template <typename ValueType, typename IndexType>
 void threshold_select(std::shared_ptr<const DefaultExecutor> exec,
-                      const matrix::Csr<ValueType, IndexType> *m,
-                      IndexType rank, Array<ValueType> &tmp,
-                      Array<remove_complex<ValueType>> &,
-                      remove_complex<ValueType> &threshold)
+                      const matrix::Csr<ValueType, IndexType>* m,
+                      IndexType rank, array<ValueType>& tmp,
+                      array<remove_complex<ValueType>>&,
+                      remove_complex<ValueType>& threshold)
 {
     auto values = m->get_const_values();
     IndexType size = m->get_num_stored_elements();
@@ -98,9 +98,9 @@ GKO_INSTANTIATE_FOR_EACH_VALUE_AND_INDEX_TYPE(
  */
 template <typename Predicate, typename ValueType, typename IndexType>
 void abstract_filter(std::shared_ptr<const DefaultExecutor> exec,
-                     const matrix::Csr<ValueType, IndexType> *m,
-                     matrix::Csr<ValueType, IndexType> *m_out,
-                     matrix::Coo<ValueType, IndexType> *m_out_coo,
+                     const matrix::Csr<ValueType, IndexType>* m,
+                     matrix::Csr<ValueType, IndexType>* m_out,
+                     matrix::Coo<ValueType, IndexType>* m_out_coo,
                      Predicate pred)
 {
     auto num_rows = m->get_size()[0];
@@ -121,7 +121,7 @@ void abstract_filter(std::shared_ptr<const DefaultExecutor> exec,
     }
 
     // build row pointers
-    components::prefix_sum(exec, new_row_ptrs, num_rows + 1);
+    components::prefix_sum_nonnegative(exec, new_row_ptrs, num_rows + 1);
 
     // second sweep: accumulate non-zeros
     auto new_nnz = new_row_ptrs[num_rows];
@@ -131,14 +131,14 @@ void abstract_filter(std::shared_ptr<const DefaultExecutor> exec,
     builder.get_value_array().resize_and_reset(new_nnz);
     auto new_col_idxs = m_out->get_col_idxs();
     auto new_vals = m_out->get_values();
-    IndexType *new_row_idxs{};
+    IndexType* new_row_idxs{};
     if (m_out_coo) {
         matrix::CooBuilder<ValueType, IndexType> coo_builder{m_out_coo};
         coo_builder.get_row_idx_array().resize_and_reset(new_nnz);
         coo_builder.get_col_idx_array() =
-            Array<IndexType>::view(exec, new_nnz, new_col_idxs);
+            make_array_view(exec, new_nnz, new_col_idxs);
         coo_builder.get_value_array() =
-            Array<ValueType>::view(exec, new_nnz, new_vals);
+            make_array_view(exec, new_nnz, new_vals);
         new_row_idxs = m_out_coo->get_row_idxs();
     }
 
@@ -163,10 +163,10 @@ void abstract_filter(std::shared_ptr<const DefaultExecutor> exec,
 
 template <typename ValueType, typename IndexType>
 void threshold_filter(std::shared_ptr<const DefaultExecutor> exec,
-                      const matrix::Csr<ValueType, IndexType> *m,
+                      const matrix::Csr<ValueType, IndexType>* m,
                       remove_complex<ValueType> threshold,
-                      matrix::Csr<ValueType, IndexType> *m_out,
-                      matrix::Coo<ValueType, IndexType> *m_out_coo, bool)
+                      matrix::Csr<ValueType, IndexType>* m_out,
+                      matrix::Coo<ValueType, IndexType>* m_out_coo, bool)
 {
     auto col_idxs = m->get_const_col_idxs();
     auto vals = m->get_const_values();
@@ -186,11 +186,11 @@ constexpr auto sample_size = bucket_count * sampleselect_oversampling;
 
 template <typename ValueType, typename IndexType>
 void threshold_filter_approx(std::shared_ptr<const DefaultExecutor> exec,
-                             const matrix::Csr<ValueType, IndexType> *m,
-                             IndexType rank, Array<ValueType> &tmp,
-                             remove_complex<ValueType> &threshold,
-                             matrix::Csr<ValueType, IndexType> *m_out,
-                             matrix::Coo<ValueType, IndexType> *m_out_coo)
+                             const matrix::Csr<ValueType, IndexType>* m,
+                             IndexType rank, array<ValueType>& tmp,
+                             remove_complex<ValueType>& threshold,
+                             matrix::Csr<ValueType, IndexType>* m_out,
+                             matrix::Coo<ValueType, IndexType>* m_out_coo)
 {
     auto vals = m->get_const_values();
     auto col_idxs = m->get_const_col_idxs();
@@ -203,7 +203,7 @@ void threshold_filter_approx(std::shared_ptr<const DefaultExecutor> exec,
                 sizeof(ValueType));
     tmp.resize_and_reset(storage_size);
     // pick and sort sample
-    auto sample = reinterpret_cast<AbsType *>(tmp.get_data());
+    auto sample = reinterpret_cast<AbsType*>(tmp.get_data());
     // assuming rounding towards zero
     auto stride = double(size) / sample_size;
     for (IndexType i = 0; i < sample_size; ++i) {
@@ -216,7 +216,7 @@ void threshold_filter_approx(std::shared_ptr<const DefaultExecutor> exec,
         sample[i] = sample[(i + 1) * sampleselect_oversampling];
     }
     // count elements per bucket
-    auto total_histogram = reinterpret_cast<IndexType *>(sample + bucket_count);
+    auto total_histogram = reinterpret_cast<IndexType*>(sample + bucket_count);
     for (IndexType bucket = 0; bucket < bucket_count; ++bucket) {
         total_histogram[bucket] = 0;
     }
@@ -241,7 +241,7 @@ void threshold_filter_approx(std::shared_ptr<const DefaultExecutor> exec,
         }
     }
     // determine splitter ranks: prefix sum over bucket counts
-    components::prefix_sum(exec, total_histogram, bucket_count + 1);
+    components::prefix_sum_nonnegative(exec, total_histogram, bucket_count + 1);
     // determine the bucket containing the threshold rank:
     // prefix_sum[bucket] <= rank < prefix_sum[bucket + 1]
     auto it = std::upper_bound(total_histogram,
@@ -266,12 +266,12 @@ GKO_INSTANTIATE_FOR_EACH_VALUE_AND_INDEX_TYPE(
 
 template <typename ValueType, typename IndexType>
 void compute_l_u_factors(std::shared_ptr<const DefaultExecutor> exec,
-                         const matrix::Csr<ValueType, IndexType> *a,
-                         matrix::Csr<ValueType, IndexType> *l,
-                         const matrix::Coo<ValueType, IndexType> *,
-                         matrix::Csr<ValueType, IndexType> *u,
-                         const matrix::Coo<ValueType, IndexType> *,
-                         matrix::Csr<ValueType, IndexType> *u_csc)
+                         const matrix::Csr<ValueType, IndexType>* a,
+                         matrix::Csr<ValueType, IndexType>* l,
+                         const matrix::Coo<ValueType, IndexType>*,
+                         matrix::Csr<ValueType, IndexType>* u,
+                         const matrix::Coo<ValueType, IndexType>*,
+                         matrix::Csr<ValueType, IndexType>* u_csc)
 {
     auto num_rows = a->get_size()[0];
     auto l_row_ptrs = l->get_const_row_ptrs();
@@ -350,12 +350,12 @@ GKO_INSTANTIATE_FOR_EACH_VALUE_AND_INDEX_TYPE(
 
 template <typename ValueType, typename IndexType>
 void add_candidates(std::shared_ptr<const DefaultExecutor> exec,
-                    const matrix::Csr<ValueType, IndexType> *lu,
-                    const matrix::Csr<ValueType, IndexType> *a,
-                    const matrix::Csr<ValueType, IndexType> *l,
-                    const matrix::Csr<ValueType, IndexType> *u,
-                    matrix::Csr<ValueType, IndexType> *l_new,
-                    matrix::Csr<ValueType, IndexType> *u_new)
+                    const matrix::Csr<ValueType, IndexType>* lu,
+                    const matrix::Csr<ValueType, IndexType>* a,
+                    const matrix::Csr<ValueType, IndexType>* l,
+                    const matrix::Csr<ValueType, IndexType>* u,
+                    matrix::Csr<ValueType, IndexType>* l_new,
+                    matrix::Csr<ValueType, IndexType>* u_new)
 {
     auto num_rows = a->get_size()[0];
     auto l_row_ptrs = l->get_const_row_ptrs();
@@ -371,7 +371,7 @@ void add_candidates(std::shared_ptr<const DefaultExecutor> exec,
     abstract_spgeam(
         a, lu, [](IndexType) { return std::pair<IndexType, IndexType>{}; },
         [](IndexType row, IndexType col, ValueType, ValueType,
-           std::pair<IndexType, IndexType> &nnzs) {
+           std::pair<IndexType, IndexType>& nnzs) {
             nnzs.first += col <= row;
             nnzs.second += col >= row;
         },
@@ -380,8 +380,8 @@ void add_candidates(std::shared_ptr<const DefaultExecutor> exec,
             u_new_row_ptrs[row] = nnzs.second;
         });
 
-    components::prefix_sum(exec, l_new_row_ptrs, num_rows + 1);
-    components::prefix_sum(exec, u_new_row_ptrs, num_rows + 1);
+    components::prefix_sum_nonnegative(exec, l_new_row_ptrs, num_rows + 1);
+    components::prefix_sum_nonnegative(exec, u_new_row_ptrs, num_rows + 1);
 
     // resize arrays
     auto l_nnz = l_new_row_ptrs[num_rows];
@@ -421,7 +421,7 @@ void add_candidates(std::shared_ptr<const DefaultExecutor> exec,
             return state;
         },
         [&](IndexType row, IndexType col, ValueType a_val, ValueType lu_val,
-            row_state &state) {
+            row_state& state) {
             auto r_val = a_val - lu_val;
             // load matching entry of L + U
             auto lpu_col = state.finished_l
