@@ -59,9 +59,9 @@ protected:
     using i_type = int;
     using CsrMtx = gko::matrix::Csr<v_type, i_type>;
     using reorder_type = gko::reorder::Rcm<v_type, i_type>;
+    using new_reorder_type = gko::experimental::reorder::Rcm<i_type>;
     using perm_type = gko::matrix::Permutation<i_type>;
 
-
     Rcm()
         : exec(gko::ReferenceExecutor::create()),
           // clang-format off
@@ -83,6 +83,17 @@ protected:
                                          {1., 0., 0., 0., 1., 0., 0., 1., 1.},
                                          {1., 0., 0., 1., 1., 0., 0., 1., 1.}},
                                         exec)),
+        p_mtx_1_lower(gko::initialize<CsrMtx>(
+                                        {{1., 0., 0., 0., 0., 0., 0., 0., 0.},
+                                         {0., 1., 0., 0., 0., 0., 0., 0., 0.},
+                                         {0., 1., 1., 0., 0., 0., 0., 0., 0.},
+                                         {1., 1., 0., 1., 0., 0., 0., 0., 0.},
+                                         {1., 0., 0., 1., 1., 0., 0., 0., 0.},
+                                         {1., 1., 1., 1., 1., 1., 0., 0., 0.},
+                                         {0., 1., 1., 1., 1., 1., 1., 0., 0.},
+                                         {1., 0., 0., 0., 1., 0., 0., 1., 0.},
+                                         {1., 0., 0., 1., 1., 0., 0., 1., 1.}},
+                                        exec)),
           // clang-format on
           rcm_factory(reorder_type::build().on(exec)),
           reorder_op_0(rcm_factory->generate(p_mtx_0)),
@@ -95,10 +106,11 @@ protected:
     std::unique_ptr<reorder_type> reorder_op_0;
     std::shared_ptr<CsrMtx> p_mtx_1;
     std::unique_ptr<reorder_type> reorder_op_1;
+    std::shared_ptr<CsrMtx> p_mtx_1_lower;
 
     static bool is_permutation(const perm_type* input_perm)
     {
-        const auto perm_size = input_perm->get_permutation_size();
+        const auto perm_size = input_perm->get_size()[0];
         auto perm_sorted = std::vector<i_type>(perm_size);
         std::copy_n(input_perm->get_const_permutation(), perm_size,
                     perm_sorted.begin());
@@ -140,4 +152,29 @@ TEST_F(Rcm, PermutesPerfectFullBand)
 }
 
 
+TEST_F(Rcm, NewInterfaceWorksOnSymmetric)
+{
+    std::vector<i_type> correct = {7, 8, 0, 4, 3, 5, 6, 1, 2};
+
+    auto permutation =
+        new_reorder_type::build().with_skip_symmetrize(true).on(exec)->generate(
+            p_mtx_1);
+
+    auto p = permutation->get_const_permutation();
+    ASSERT_TRUE(std::equal(p, p + correct.size(), correct.begin()));
+}
+
+
+TEST_F(Rcm, NewInterfaceWorksOnNonsymmetric)
+{
+    std::vector<i_type> correct = {7, 8, 0, 4, 3, 5, 6, 1, 2};
+
+    auto permutation =
+        new_reorder_type::build().on(exec)->generate(p_mtx_1_lower);
+
+    auto p = permutation->get_const_permutation();
+    ASSERT_TRUE(std::equal(p, p + correct.size(), correct.begin()));
+}
+
+
 }  // namespace
