@@ -54,6 +54,9 @@ OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 #include "core/test/utils.hpp"
 
 
+GKO_BEGIN_DISABLE_DEPRECATION_WARNINGS
+
+
 namespace {
 
 
@@ -110,11 +113,9 @@ protected:
           solver_factory(
               Bicgstab::build()
                   .with_criteria(
-                      gko::stop::Iteration::build().with_max_iters(100u).on(
-                          exec),
+                      gko::stop::Iteration::build().with_max_iters(100u),
                       gko::stop::ResidualNorm<value_type>::build()
-                          .with_reduction_factor(r<value_type>::value)
-                          .on(exec))
+                          .with_reduction_factor(r<value_type>::value))
                   .on(exec)),
           tol{r<value_type>::value}
     {
@@ -570,3 +571,6 @@ TYPED_TEST(ScaledReordered, SolvesMultipleRhs)
 
 
 }  // namespace
+
+
+GKO_END_DISABLE_DEPRECATION_WARNINGS
