@@ -289,30 +289,26 @@ protected:
                                  .on(exec)),
           smoother_factory(gko::give(
               Smoother::build()
-                  .with_solver(
-                      InnerSolver::build().with_max_block_size(1u).on(exec))
+                  .with_solver(InnerSolver::build().with_max_block_size(1u))
                   .with_criteria(
-                      gko::stop::Iteration::build().with_max_iters(1u).on(exec))
+                      gko::stop::Iteration::build().with_max_iters(1u))
                   .on(exec))),
           coarsest_factory(
               CoarsestSolver::build()
                   .with_criteria(
-                      gko::stop::Iteration::build().with_max_iters(4u).on(exec),
-                      gko::stop::Time::build()
-                          .with_time_limit(std::chrono::seconds(6))
-                          .on(exec),
+                      gko::stop::Iteration::build().with_max_iters(4u),
+                      gko::stop::Time::build().with_time_limit(
+                          std::chrono::seconds(6)),
                       gko::stop::ResidualNorm<value_type>::build()
                           .with_baseline(gko::stop::mode::initial_resnorm)
-                          .with_reduction_factor(r<value_type>::value)
-                          .on(exec))
+                          .with_reduction_factor(r<value_type>::value))
                   .on(exec)),
           coarsestnext_factory(
               CoarsestNextSolver::build()
                   .with_criteria(
-                      gko::stop::Iteration::build().with_max_iters(4u).on(exec),
-                      gko::stop::Time::build()
-                          .with_time_limit(std::chrono::seconds(6))
-                          .on(exec))
+                      gko::stop::Iteration::build().with_max_iters(4u),
+                      gko::stop::Time::build().with_time_limit(
+                          std::chrono::seconds(6)))
                   .on(exec)),
           rp_factory(DummyRPFactory::build().on(exec)),
           lo_factory(DummyFactory::build().on(exec)),
@@ -357,14 +353,12 @@ protected:
                 .with_mid_case(gko::solver::multigrid::mid_smooth_type::both)
                 .with_mg_level(coarse_factory)
                 .with_criteria(
-                    gko::stop::Iteration::build().with_max_iters(4u).on(exec),
-                    gko::stop::Time::build()
-                        .with_time_limit(std::chrono::seconds(6))
-                        .on(exec),
+                    gko::stop::Iteration::build().with_max_iters(4u),
+                    gko::stop::Time::build().with_time_limit(
+                        std::chrono::seconds(6)),
                     gko::stop::ResidualNorm<value_type>::build()
                         .with_baseline(gko::stop::mode::initial_resnorm)
-                        .with_reduction_factor(r<value_type>::value)
-                        .on(exec))
+                        .with_reduction_factor(r<value_type>::value))
                 .with_cycle(cycle)
                 .with_min_coarse_rows(1u)
                 .on(exec));
@@ -382,14 +376,12 @@ protected:
                 .with_mid_case(gko::solver::multigrid::mid_smooth_type::both)
                 .with_mg_level(coarse_factory, coarsenext_factory)
                 .with_criteria(
-                    gko::stop::Iteration::build().with_max_iters(200u).on(exec),
-                    gko::stop::Time::build()
-                        .with_time_limit(std::chrono::seconds(100))
-                        .on(exec),
+                    gko::stop::Iteration::build().with_max_iters(200u),
+                    gko::stop::Time::build().with_time_limit(
+                        std::chrono::seconds(100)),
                     gko::stop::ResidualNorm<value_type>::build()
                         .with_baseline(gko::stop::mode::initial_resnorm)
-                        .with_reduction_factor(r<value_type>::value)
-                        .on(exec))
+                        .with_reduction_factor(r<value_type>::value))
                 .with_cycle(cycle)
                 .with_min_coarse_rows(1u)
                 .on(exec));
@@ -413,9 +405,7 @@ protected:
                     gko::matrix::IdentityFactory<value_type>::create(exec))
                 .with_post_uses_pre(false)
                 .with_mid_case(mid_case)
-                .with_criteria(
-                    gko::stop::Iteration::build().with_max_iters(1u).on(
-                        this->exec))
+                .with_criteria(gko::stop::Iteration::build().with_max_iters(1u))
                 .with_cycle(cycle)
                 .with_min_coarse_rows(1u)
                 .on(this->exec));
@@ -435,9 +425,7 @@ protected:
                 .with_coarsest_solver(this->lo_factory)
                 .with_post_uses_pre(true)
                 .with_mid_case(mid_case)
-                .with_criteria(
-                    gko::stop::Iteration::build().with_max_iters(1u).on(
-                        this->exec))
+                .with_criteria(gko::stop::Iteration::build().with_max_iters(1u))
                 .with_cycle(cycle)
                 .with_min_coarse_rows(1u)
                 .on(this->exec));
@@ -1273,8 +1261,7 @@ TYPED_TEST(Multigrid, ZeroGuessIgnoresInput)
             .with_coarsest_solver(this->coarsest_factory)
             .with_max_levels(2u)
             .with_mg_level(this->coarse_factory)
-            .with_criteria(
-                gko::stop::Iteration::build().with_max_iters(1u).on(this->exec))
+            .with_criteria(gko::stop::Iteration::build().with_max_iters(1u))
             .with_min_coarse_rows(1u);
     auto normal_mg = common_part
                          .with_default_initial_guess(
