@@ -65,6 +65,7 @@ class ResidualNorm : public CommonTestFixture {
 protected:
     using Mtx = gko::matrix::Dense<T>;
     using NormVector = gko::matrix::Dense<gko::remove_complex<T>>;
+    using ValueType = T;
 
     ResidualNorm()
     {
@@ -110,6 +111,40 @@ TYPED_TEST(ResidualNorm, CanIgorneResidualNorm)
                  gko::NotSupported);
 }
 
+TYPED_TEST(ResidualNorm, CheckIfResZeroConverges)
+{
+    using Mtx = typename TestFixture::Mtx;
+    using NormVector = typename TestFixture::NormVector;
+    using T = typename TestFixture::ValueType;
+    using mode = gko::stop::mode;
+    std::shared_ptr<gko::LinOp> mtx = gko::initialize<Mtx>({1.0}, this->exec);
+    std::shared_ptr<gko::LinOp> rhs = gko::initialize<Mtx>({0.0}, this->exec);
+    std::shared_ptr<gko::LinOp> x = gko::initialize<Mtx>({0.0}, this->exec);
+    std::shared_ptr<gko::LinOp> res_norm =
+        gko::initialize<NormVector>({0.0}, this->exec);
+
+    for (auto baseline :
+         {mode::rhs_norm, mode::initial_resnorm, mode::absolute}) {
+        gko::remove_complex<T> factor =
+            (baseline == mode::absolute) ? 0.0 : r<T>::value;
+        auto criterion = gko::stop::ResidualNorm<T>::build()
+                             .with_reduction_factor(factor)
+                             .with_baseline(baseline)
+                             .on(this->exec)
+                             ->generate(mtx, rhs, x.get(), nullptr);
+        constexpr gko::uint8 RelativeStoppingId{1};
+        bool one_changed{};
+        gko::array<gko::stopping_status> stop_status(this->ref, 1);
+        stop_status.get_data()[0].reset();
+        stop_status.set_executor(this->exec);
+
+        EXPECT_TRUE(criterion->update().residual_norm(res_norm).check(
+            RelativeStoppingId, true, &stop_status, &one_changed));
+        stop_status.set_executor(this->ref);
+        EXPECT_TRUE(stop_status.get_data()[0].has_converged());
+        EXPECT_TRUE(one_changed);
+    }
+}
 
 TYPED_TEST(ResidualNorm, WaitsTillResidualGoal)
 {
@@ -522,6 +557,7 @@ class ImplicitResidualNorm : public CommonTestFixture {
 protected:
     using Mtx = gko::matrix::Dense<T>;
     using NormVector = gko::matrix::Dense<gko::remove_complex<T>>;
+    using ValueType = T;
 
     ImplicitResidualNorm()
     {
@@ -538,6 +574,42 @@ TYPED_TEST_SUITE(ImplicitResidualNorm, gko::test::ValueTypes,
                  TypenameNameGenerator);
 
 
+TYPED_TEST(ImplicitResidualNorm, CheckIfResZeroConverges)
+{
+    using Mtx = typename TestFixture::Mtx;
+    using T = typename TestFixture::ValueType;
+    using mode = typename gko::stop::mode;
+    std::shared_ptr<gko::LinOp> mtx = gko::initialize<Mtx>({1.0}, this->exec);
+    std::shared_ptr<gko::LinOp> rhs = gko::initialize<Mtx>({0.0}, this->exec);
+    std::shared_ptr<gko::LinOp> x = gko::initialize<Mtx>({0.0}, this->exec);
+    std::shared_ptr<gko::LinOp> implicit_sq_res_norm =
+        gko::initialize<Mtx>({0.0}, this->exec);
+
+    for (auto baseline :
+         {mode::rhs_norm, mode::initial_resnorm, mode::absolute}) {
+        gko::remove_complex<T> factor =
+            (baseline == mode::absolute) ? 0.0 : r<T>::value;
+        auto criterion = gko::stop::ImplicitResidualNorm<T>::build()
+                             .with_reduction_factor(factor)
+                             .with_baseline(baseline)
+                             .on(this->exec)
+                             ->generate(mtx, rhs, x.get(), nullptr);
+        constexpr gko::uint8 RelativeStoppingId{1};
+        bool one_changed{};
+        gko::array<gko::stopping_status> stop_status(this->ref, 1);
+        stop_status.get_data()[0].reset();
+        stop_status.set_executor(this->exec);
+
+        EXPECT_TRUE(
+            criterion->update()
+                .implicit_sq_residual_norm(implicit_sq_res_norm)
+                .check(RelativeStoppingId, true, &stop_status, &one_changed));
+        stop_status.set_executor(this->ref);
+        EXPECT_TRUE(stop_status.get_data()[0].has_converged());
+        EXPECT_TRUE(one_changed);
+    }
+}
+
 TYPED_TEST(ImplicitResidualNorm, WaitsTillResidualGoal)
 {
     using T = TypeParam;
