@@ -44,34 +44,10 @@ OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 #include <gtest/gtest.h>
 
 
-#ifdef GKO_COMPILING_CUDA
-
-#include "cuda/base/device.hpp"
-
-class CudaEnvironment : public ::testing::Environment {
-public:
-    void TearDown() override { gko::kernels::cuda::reset_device(0); }
-};
-
-testing::Environment* cuda_env =
-    testing::AddGlobalTestEnvironment(new CudaEnvironment);
-
-#endif
+#include <ginkgo/core/base/stream.hpp>
 
 
-#ifdef GKO_COMPILING_HIP
-
-#include "hip/base/device.hpp"
-
-class HipEnvironment : public ::testing::Environment {
-public:
-    void TearDown() override { gko::kernels::hip::reset_device(0); }
-};
-
-testing::Environment* hip_env =
-    testing::AddGlobalTestEnvironment(new HipEnvironment);
-
-#endif
+#include "core/test/gtest/resources.hpp"
 
 
 #if GINKGO_COMMON_SINGLE_MODE
@@ -106,8 +82,9 @@ inline void init_executor(std::shared_ptr<gko::ReferenceExecutor> ref,
         if (gko::CudaExecutor::get_num_devices() == 0) {
             throw std::runtime_error{"No suitable CUDA devices"};
         }
-        exec = gko::CudaExecutor::create(0, ref, false,
-                                         gko::default_cuda_alloc_mode, stream);
+        exec = gko::CudaExecutor::create(
+            ResourceEnvironment::cuda_device_id, ref,
+            std::make_shared<gko::CudaAllocator>(), stream);
     }
 }
 
@@ -119,8 +96,9 @@ inline void init_executor(std::shared_ptr<gko::ReferenceExecutor> ref,
     if (gko::HipExecutor::get_num_devices() == 0) {
         throw std::runtime_error{"No suitable HIP devices"};
     }
-    exec = gko::HipExecutor::create(0, ref, false, gko::default_hip_alloc_mode,
-                                    stream);
+    exec =
+        gko::HipExecutor::create(ResourceEnvironment::hip_device_id, ref,
+                                 std::make_shared<gko::HipAllocator>(), stream);
 }
 
 
@@ -128,7 +106,8 @@ inline void init_executor(std::shared_ptr<gko::ReferenceExecutor> ref,
                           std::shared_ptr<gko::DpcppExecutor>& exec)
 {
     if (gko::DpcppExecutor::get_num_devices("gpu") > 0) {
-        exec = gko::DpcppExecutor::create(0, ref, "gpu");
+        exec = gko::DpcppExecutor::create(ResourceEnvironment::sycl_device_id,
+                                          ref, "gpu");
     } else if (gko::DpcppExecutor::get_num_devices("cpu") > 0) {
         exec = gko::DpcppExecutor::create(0, ref, "cpu");
     } else {
@@ -146,14 +125,24 @@ public:
 #endif
     using index_type = int;
 
-    CommonTestFixture() : ref{gko::ReferenceExecutor::create()}
+    CommonTestFixture()
+        :
+#if defined(GKO_TEST_NONDEFAULT_STREAM) && defined(GKO_COMPILING_CUDA)
+          stream(ResourceEnvironment::cuda_device_id),
+#endif
+#if defined(GKO_TEST_NONDEFAULT_STREAM) && defined(GKO_COMPILING_HIP)
+          stream(ResourceEnvironment::hip_device_id),
+#endif
+          ref{gko::ReferenceExecutor::create()}
     {
-#if defined(GKO_TEST_NONDEFAULT_STREAM) && \
-    (defined(GKO_COMPILING_CUDA) || defined(GKO_COMPILING_HIP))
+#if defined(GKO_COMPILING_CUDA) || defined(GKO_COMPILING_HIP)
         init_executor(ref, exec, stream.get());
 #else
         init_executor(ref, exec);
 #endif
+        // set device-id test-wide since some test call device
+        // kernels directly
+        guard = exec->get_scoped_device_id_guard();
     }
 
     void TearDown() final
@@ -163,16 +152,15 @@ public:
         }
     }
 
-#ifdef GKO_TEST_NONDEFAULT_STREAM
 #ifdef GKO_COMPILING_CUDA
     gko::cuda_stream stream;
 #endif
 #ifdef GKO_COMPILING_HIP
     gko::hip_stream stream;
-#endif
 #endif
     std::shared_ptr<gko::ReferenceExecutor> ref;
     std::shared_ptr<gko::EXEC_TYPE> exec;
+    gko::scoped_device_id_guard guard;
 };
 
 
